/*
    Copyright (C) 2000 DFN-FWL (http://www.fwl.dfn.de/fwl)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 * $Id: analyze_header.h,v 1.10 2000/04/16 15:25:36 gellert Exp $
 *
 * $Log: analyze_header.h,v $
 * Revision 1.10  2000/04/16 15:25:36  gellert
 * removed redefinitions of output_string and error_string.
 *
 * Revision 1.9  2000/03/07 15:06:46  goldbach
 * added GPL info
 *
 * Revision 1.8  2000/01/25 15:03:25  gellert
 * Now the name and content of the actual headerline are processed
 * in the tct->inbuf. No copying to local variables (that are too
 * small for long lines (eg "Referer: thousands_of_bytes").
 *
 * Revision 1.7  1999/11/24 18:16:22  gellert
 * new function "process_proxy_connection".
 *
 * Revision 1.6  1999/11/11 15:48:23  gellert
 * Added declaration of "process_content_length".
 *
 * Revision 1.5  1999/10/26 14:09:22  gellert
 * new function: copy_fieldname (copies just the fieldname,
 * but not the value (= content) of the field.
 *
 * Revision 1.4  1999/07/22 14:27:59  goldbach
 * minor cleanup for logmsg() (again)
 *
 * Revision 1.3  1999/05/12 16:04:19  goldbach
 * major clean-up session
 *
 * Revision 1.2  1999/03/30 12:30:05  goldbach
 * 1st surfable version
 *
 * Revision 1.1  1999/02/18 17:33:15  goldbach
 * *** empty log message ***
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "mythreads.h"

/*
 * name -- field name
 * action -- function to call if this header field is found
 */

struct header_action
{
    char *name;
    int (*action)(char *, TCT*);
};


/*
 * Konstanten fuer Protokollierung.
 * FIXME
 *
 * ich haette hier gerne 'const int' genommen, aber der gcc sagt:
 * 'initializer element for `header_actions[0].loglevel' is not constant'
 * Der cc sagt aehnliches.
 * gg 28.01.99
 */
#define LOGLEVEL_NONE 0
#define LOGLEVEL_VERBOSE 1

/*
 * builtin http version. We expect to get this and set it if
 * this is not the case.
 */
#define BUILTIN_HTTP_MAJOR 1
#define BUILTIN_HTTP_MINOR 0

/*
 * mime types. only those we really recognize, all other
 * types are of MT_OTHER
 */
#define MT_OTHER 0
#define MT_HTML 1

/* skip this line (don't send it) */
int skip_headerline(char *, TCT *);

/* verbatim output */
int copy_headerline(char *, TCT *);

/* copy just the fieldname, but no content */
int copy_fieldname(char *, TCT *);

/* copy the 'Content-Length' entry, if we are using
 * the POST method
  */
int process_content_length(char *, TCT *);

/* replace attribute of proxy-connection "keep-alive" with "close" */
int process_proxy_connection(char *, TCT *);

/* process header (w/o request and response line) */
int process_http_header(TCT *, int);

char *find_eol(char *, int, int);

