/*
 * $Id: args.h,v 1.2 2001/05/04 12:05:42 gellert Exp $
 *
 * args.c -- get command line arguments
 *
 * $Log: args.h,v $
 * Revision 1.2  2001/05/04 12:05:42  gellert
 * Changed the connect-timeout from 10 to 30 seconds.
 *
 * Revision 1.1  2001/03/01 16:06:29  glauschwuffel
 * argument processing module. added option 'f' (config file) and 'V'
 * (logging facility
 *
 */

#ifndef __ARGS_H__
#define __ARGS_H__

#include <sys/param.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/syslog.h>
#include <netdb.h> /* MAXHOSTNAMELEN */

#define DEFAULT_COPY_SIZE 8192
#define DEFAULT_SOCKET_SIZE 0
#define DEFAULT_SERVER_PORT 1080
#define DEFAULT_NUMBER_OF_THREADS 50 /* spawn 50 threads on startup */
#define DEFAULT_LOG_LEVEL LOG_NOTICE
#define DEFAULT_REAL_PROXY_FLAG 1
#define DEFAULT_DEADLOCK_CHECKER_INTERVALL 30
#define DEFAULT_RECV_DURATION 30 /* recv() may need a few seconds */
#define DEFAULT_CONFIG_FILE "httpf.cfg"
#define DEFAULT_LOG_FACILITY 0

#define MAX_CONFIG_FILE_NAME_LENGTH 80

#define	MAXPACKETLEN	65536

#define ARGUMENT_STRING "?hp:m:ns:S:t:d:D:v:V:f:"

typedef struct args {
  int number_of_threads;
  int server_socket_size;
  int client_socket_size;
  int copy_size;
  int daemon_flag;
  int server_port;
  struct sockaddr_in destination_address;
  int log_level;
  int real_proxy_flag;
  char destination_host[MAXHOSTNAMELEN];
  int deadlock_checker_intervall;
  int recv_duration;
  char config_file[MAX_CONFIG_FILE_NAME_LENGTH];
  int log_facility;
} ARGS;

void process_args(int,char**);
void set_default_args(void);

void set_resource_limit(void);

#endif
