/*
    Copyright (C) 2000 DFN-FWL (http://www.fwl.dfn.de/fwl)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 * $Id: cfglists.c,v 1.5 2000/06/21 18:50:35 glauschwuffel Exp $
 *
 * $Log: cfglists.c,v $
 * Revision 1.5  2000/06/21 18:50:35  glauschwuffel
 * proto_allowed() now gets the context and logs nicely.
 *
 * Revision 1.4  2000/06/19 21:43:03  glauschwuffel
 * memchr() is now called instead of strnchr.
 * Adpated log levels.
 * includes log.h instead of misc.h
 *
 * Revision 1.3  2000/05/07 14:57:02  glauschwuffel
 * added init. of local var prev_point to kill the gcc warning
 *
 * Revision 1.2  2000/04/24 21:22:47  gellert
 * fiddled around with inet address. now client_allowed works on linux
 * and solaris.
 *
 * Revision 1.1  2000/04/24 19:44:34  gellert
 * Added new feature: clients entry in httpf.cfg. Only configured
 * clients are allowed to connect to httpf.
 * The handling of the lists of protocols and the lists
 * of allowed clients are now handled in cfglists.c
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>
#include <ctype.h>
#include <netinet/in.h>
#include "log.h"
#include "cfglists.h"
#include "read_config.h"

struct proto_list {
    struct	proto_list *next;	/* next entry */
    char	*proto;			/* the value */
};

struct proto_list *global_protos = NULL;


struct client_list {
    struct client_list *next;
    uint32_t	lower;			/* lower limit */
    uint32_t	upper;			/* upper limit */
};

struct client_list *global_clients = NULL;



/* proto_insert: insert an allowed protocol */
int proto_insert(char *proto) {
  struct   proto_list   *proto_point;
  
    if (global_protos == NULL) {
      global_protos = (struct proto_list *) malloc(sizeof(*proto_point));
      proto_point = global_protos;
      proto_point->next = NULL;
    } else { /* there is already at least one proto */
      proto_point = global_protos;
      while (proto_point->next != NULL) {
        proto_point = proto_point->next;
        }
      proto_point->next = (struct proto_list *) malloc(sizeof(*proto_point));
      proto_point = proto_point->next;
      proto_point->next = NULL;
    } /* now proto_point points to a new allocated cell */
    if ((proto_point->proto = strdup(proto)) == NULL) {
      return(-1);
    } else {
      return(0);
    }
}

/* proto_allowed: find a proto in the entry for an attribute */
int proto_allowed(char *proto, TCT*tct) {
  struct   proto_list   *proto_point;
  
  if (global_protos == NULL) {
    /* no entries configured */
    tct->error=NO_PROTOCOLS_CONFIGURED;
    logmsg(LOG_ERR, "[%d] No protocol entries configured.", tct->tid);
    return FAILURE;
  }
    
  proto_point = global_protos;
  while (proto_point != NULL) {
    if (strlen(proto) > strlen(proto_point->proto)) {
      if ((strncasecmp(proto_point->proto, proto, strlen(proto_point->proto))==0)
	  && (proto[strlen(proto_point->proto)] == ':')) {
	return OKAY; 	/* protocol was found! */
      }
    }
    proto_point = proto_point->next;
  } /* while */

  logmsg(LOG_NOTICE, "[%d] Protocol '%s' not allowed",
	 tct->tid, proto);
  tct->error=UNALLOWED_PROTOCOL;
  return FAILURE;
}

int process_protos(char *protocol) {
  char	*temp;

     if (strncmp(protocol, "", 1)==0) {
       return(0);	/* end of recursion */
     } else {
       temp = memchr(protocol, ',', strlen(protocol));
       if (temp == NULL) {
         /* ok, this is the last word in the string */
         return(proto_insert(protocol));
       } else {
         /* this is not the last word, so copy the first word
            and process the rest of the string */
	 *temp = 0;	/* mark the end of the word */
	 proto_insert(protocol);
	 *temp = ',';	/* restore the old string */
	 return (process_protos(temp+1));
       }
     }
}   

/* string_to_client: convert config string to client entry */
int string_to_client(char *configstring, struct client_list *entry) {
  int result;
  char *strnames[4];
  int lower[4] = { 0, 0, 0, 0 };
  int upper[4] = { 255,255,255,255 };
  int count;
  char *find_point, *old_point;

  strnames[0] = NULL; strnames[1] = NULL;
  strnames[2] = NULL; strnames[3] = NULL; 
  find_point = configstring;

  for (result = 0; result < 3; result++) {
    old_point = find_point;
    find_point = memchr(find_point, '.', strlen(find_point));
    if (find_point == NULL) {
      logmsg(LOG_NOTICE, "Wrong dotted notation for client");
      return(-1);
      }
    find_point[0] = 0;
    strnames[result]= old_point;
    find_point++;
    }

  old_point = find_point;
  strnames[result] = old_point;
  result++;
  
  for (count = 0; count < result; count++) {
      if (strncmp(strnames[count], "*", 2) == 0) {
          upper[count] = 255;
          lower[count] = 0;
      } else {
          upper[count] = atoi(strnames[count]);
	  lower[count] = upper[count];
      }
    } /* end of for */
  
  for (count = 0; count < 4; count++) {
      upper[count] &= 0xff;
      lower[count] &= 0xff;
    }
  
  entry->upper = (upper[0] << 24) | (upper[1] << 16) |
  		 (upper[2] << 8) | (upper[3]);
  entry->lower = (lower[0] << 24) | (lower[1] << 16) |
  		 (lower[2] << 8) | (lower[3]);

  logmsg(LOG_DEBUG, "string_to_client: upper: %x, lower: %x\n", entry->upper, entry->lower);
  return(0);
}

/* client_insert: insert an allowed protocol */
int client_insert(char *configstring) {
  struct   client_list   *client_point, *prev_point=NULL;

  if (global_clients == NULL) {
	global_clients = (struct client_list *) malloc(sizeof(*client_point));
	if (global_clients == NULL) return(-1);
	client_point = global_clients;
	client_point->next = NULL;
	prev_point = NULL;
  } else { /* there is already at least one proto */
	client_point = global_clients;
	while (client_point->next != NULL) {
	  prev_point = client_point;
	  client_point = client_point->next;
	}
	client_point->next = (struct client_list *) malloc(sizeof(*client_point));
	if (client_point->next == NULL) return(-1);
	client_point = client_point->next;
	client_point->next = NULL;
  } /* now client_point points to a new allocated cell */

  if (string_to_client(configstring, client_point) != 0) {
	free(client_point);
	if (prev_point != NULL) {
	  prev_point->next = NULL;
	}
	return(-1);
  }

  return(0);
}

/* client_allowed: find a proto in the entry for an attribute */
int client_allowed(uint32_t address) {
  struct   client_list   *client_point;
  
  if (global_clients == NULL) {
    /* no entries configured */
    logmsg(LOG_ERR, "No client entries configured.");
    return(0);
    }
    
    client_point = global_clients;
    logmsg(LOG_DEBUG, "global_clients: %x", client_point);
    while (client_point != NULL) {
      logmsg(LOG_DEBUG, "client_allowed: ip %x, check low %x, hi %x",
      	ntohl(address), client_point->lower, client_point->upper);
      if ((client_point->lower <= ntohl(address)) &&
          (client_point->upper >= ntohl(address))) {
      		return(1);  /* client entry found */
          }
      client_point = client_point->next;
      } /* end of while */

  logmsg(LOG_NOTICE, "Unknown client");
  return(0);
}

int process_clients(char *clients) {
  char	*temp;

     if (strncmp(clients, "", 1)==0) {
       return(0);	/* end of recursion */
     } else {
       temp = memchr(clients, ',', strlen(clients));
       if (temp == NULL) {
         /* ok, this is the last word in the string */
         return(client_insert(clients));
       } else {
         /* this is not the last word, so copy the first word
            and process the rest of the string */
	 *temp = 0;	/* mark the end of the word */
	 client_insert(clients);
	 *temp = ',';	/* restore the old string */
	 return (process_clients(temp+1));
       }
     }
}   

