/*
    Copyright (C) 2000 DFN-FWL (http://www.fwl.dfn.de/fwl)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/* copy-pkts.c                          el 26.11.92 */
/*
 *
 * $Id: copy-pkts.c,v 1.24 2001/03/03 17:25:51 glauschwuffel Exp $
 * $Log: copy-pkts.c,v $
 * Revision 1.24  2001/03/03 17:25:51  glauschwuffel
 * The check for binary data declared as text/html is now available on
 * Linux, too (protected unsafe rand() with semaphore).
 *
 * Expanded the error page when binary data is found a bit:
 * the contents surrounding the suspicious byte is now displayed.
 *
 * Revision 1.23  2001/03/01 15:53:06  glauschwuffel
 * Timeout of select() went down from 60*60*2 seconds to args.recv_duration() :)
 *
 * Revision 1.22  2000/06/19 22:06:33  glauschwuffel
 * Major overhaul of the HTML parser. Now we have something like a
 * state machine (yes, I know that I should've done it that way
 * in the first place ;)).
 *
 * Split up some modules (misc.c, copy_pkts.c). filter_html.c
 * is much more cleaner now. I think that everyone should be able
 * to understand how the HTML parser works now. Especially
 * the skipping of comments is easy to get now ;)
 *
 * This seems to be getting cleaner...
 *
 * Revision 1.21  2000/06/09 14:06:35  glauschwuffel
 * moved insertion of our nifty identification string from filter_html() to copy-pkts().
 * Now it is inserted only *once* in each document :)
 *
 * Revision 1.20  2000/05/07 15:05:04  glauschwuffel
 * added init. of context flags end_to_search and more_input_data. added include ftype_check
 *
 * Revision 1.19  2000/03/07 15:06:48  goldbach
 * added GPL info
 *
 * Revision 1.18  2000/03/01 17:43:55  goldbach
 * introduced some constants for read and copy-pkts
 *
 * Revision 1.17  1999/11/30 16:57:52  gellert
 * skip flag: initialization moved to beginning of copy-pakets. read_header: find_empty_line is called with the length of the read block, not the full HTTP_BUFSIZE.
 *
 * Revision 1.16  1999/11/11 16:50:47  gellert
 * Changed a log message.
 *
 * Revision 1.15  1999/11/11 15:50:33  gellert
 * Changed the processing (= reading) of client requests.
 * Now the bodies are copied as well and "POST" works.
 *
 * Revision 1.14  1999/09/09 15:40:25  gellert
 * if there is no body following directly after the header
 * (in the first datagram) then the proxy now tries to get
 * the next datagram. because squid often splits the packets
 * and sends header in body in different datagrams.
 *
 * Revision 1.13  1999/08/04 12:00:52  gellert
 * Adapted to the change of typecheck (tct->mime_type is no a *char).
 * When an empty body arrives from the server the header is send
 * immediately to the client and that's it. In previous versions
 * the type check failed for all cached gifs etc, because the body
 * of the server response has size 0 (in the header is the response
 * line: 304 Not Modified).
 *
 * Revision 1.12  1999/07/22 15:35:03  goldbach
 * yet again cleanup for logmsg()
 *
 * Revision 1.11  1999/07/22 14:28:00  goldbach
 * minor cleanup for logmsg() (again)
 *
 * Revision 1.10  1999/07/15 16:55:17  gellert
 * Ok, nun inverse Logik. Wenn Mimetype NICHT html, dann
 * durch den Typecheck. Sonst filtern. Vielleicht noch mal
 * testen, was bei Text passiert!!!
 *
 * Revision 1.9  1999/06/09 20:33:52  gellert
 * new version. now the parsing of tags and attributes works.
 * and it still works if the client cancels the transmission
 * (stop button in browser).
 *
 * Revision 1.8  1999/05/27 16:47:59  goldbach
 * minor cleanup for logmsg()
 *
 * Revision 1.7  1999/05/27 11:29:34  gellert
 * well, now it does what we want. just small mistakes (eg end of
 * comments) are done.
 *
 * Revision 1.6  1999/05/12 16:04:20  goldbach
 * major clean-up session
 *
 * Revision 1.5  1999/03/30 12:30:07  goldbach
 * 1st surfable version
 *
 * Revision 1.4  1999/02/16 17:43:56  gellert
 * now it works. up to now the header is read and passed on to the
 * server, the other direction is just copied through.
 *
 * Revision 1.3  1999/02/11 14:38:48  gellert
 * ein paar alte defines ausgemerzt, neue kommentare fuer kommende
 * funktionen schon mal eingefuegt.
 *
 * Revision 1.2  1999/02/09 13:39:28  gellert
 * removed all the "ifdef ssl/des3/arc4" - parts, so the code is now
 * a little bit simpler & shorter :-)
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <syslog.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include "log.h"
#include "mythreads.h"
#include "process_client.h"
#include "process_server.h"
#include "args.h"

extern ARGS args;


#ifndef MAX
#define MAX(a,b) ((a>b)?a:b)
#endif

int copy_pkts (TCT *tct) {
  int 	nfds = MAX(tct->insocket,tct->outsocket)+1;   
  fd_set 	fds;	/* fuer select: liste mit sockin + sockout */
  static struct timeval	tout;
  int result;
  
  FD_ZERO(&fds);

  while (1) {
    int num_fds; /* select () */
    
    tout.tv_sec = args.recv_duration;
    tout.tv_usec = 0;

    FD_SET(tct->insocket, &fds);
    FD_SET(tct->outsocket, &fds);

    /*
     * reset skip flag. we reset it here regardless of wether select()
     * returns with a call from the client or the server.
     * Since the http-header functions use output_string() and
     * the skip_flag is used in there we reset it here.
     */
    tct->skip_flag=0;
    
    num_fds=select(nfds, &fds, NULL, NULL, &tout);
    if (num_fds > 0) {
      /* request? */
      if (FD_ISSET(tct->insocket, &fds)) {
	result = process_client(tct);
	if (result != OKAY) {
	  logmsg(LOG_DEBUG, "[%d] process_client() failed", tct->tid);
	  return result;
	} else {
	  logmsg(LOG_DEBUG, "[%d] Client request processed successfully", tct->tid);
	}
      }
      
      /* response? */
      if (FD_ISSET(tct->outsocket, &fds)) {
	result = process_server(tct);
	if (result != OKAY) {
	  logmsg(LOG_NOTICE, "[%d] Server data not sent completely.", tct->tid);
	  return result;
	}
	break; /* eof/everything ok: next select */
      }
    } else if (num_fds == 0) { /* timeout */
      logmsg(LOG_ERR,
	     "[%d] Select on server and client sockets failed with timeout.",
	     tct->tid);
      tct->error=SOCKET;
      return FAILURE;
    } else {
      logmsg(LOG_ERR,
	     "[%d] Select on server and client sockets failed with errno %d.",
	     tct->tid, errno);
      tct->error=SOCKET;
      return FAILURE;
    }
  }
  
  return 0; 
}




