/*
    Copyright (C) 2000 DFN-FWL (http://www.fwl.dfn.de/fwl)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 * $Id: filter_html.c,v 1.42 2001/03/01 15:55:22 glauschwuffel Exp $
 *
 * $Log: filter_html.c,v $
 * Revision 1.42  2001/03/01 15:55:22  glauschwuffel
 * adapted for changes with read.c
 *
 * Revision 1.41  2000/06/21 18:49:27  glauschwuffel
 * Always send the contents of the output buffer now, not only if
 * it was parsed completely. This ensures that the user sees at least
 * some part of the document even if there was an error...
 * Adjusted log messages.
 *
 * Revision 1.40  2000/06/19 22:06:33  glauschwuffel
 * Major overhaul of the HTML parser. Now we have something like a
 * state machine (yes, I know that I should've done it that way
 * in the first place ;)).
 *
 * Split up some modules (misc.c, copy_pkts.c). filter_html.c
 * is much more cleaner now. I think that everyone should be able
 * to understand how the HTML parser works now. Especially
 * the skipping of comments is easy to get now ;)
 *
 * This seems to be getting cleaner...
 *
 * Revision 1.39  2000/06/09 14:54:47  glauschwuffel
 * minor cleanup and comments
 *
 * Revision 1.38  2000/06/09 14:07:25  glauschwuffel
 * moved insertion of our nifty identification string from filter_html() to copy-pkts().
 * Now it is inserted only *once* in each document :)
 *
 * Revision 1.37  2000/05/09 14:38:05  gellert
 * if the last tag (normally </HTML>) has no following newline,
 * the inbuff is fully processed, the next read fails and the
 * proxy didn't send the already parsed outbuf. Now he does.
 *
 * Revision 1.36  2000/05/08 13:15:16  glauschwuffel
 * removed log message printing httpf_id
 *
 * Revision 1.35  2000/05/07 15:15:41  glauschwuffel
 * made functions local. added insertion of httpf_id at the start of each processed document. removed get_end_of_tag\(\). local var eot in handle_tag\(\) was unused. dragged skipping of comments into handle_tag\(\). introduced symbolic constants :\). changed handle_tag\(\) to fit new processing requirements. replaced another strncpy by memmove. introduced context flag more_input_data in  read_body_after_broken_tag.
 *
 * Revision 1.34  2000/04/24 19:52:04  gellert
 * BUGFIX: if we had an incomplete tag in the buffer, we did a
 * recv and assumed, that the buffer then is full. this is not
 * true if the server has a small segment size and we did just
 * get one segment til the recv. than the tag may still be
 * incomplete and we throw it away. now we recv until the buffer
 * is full (or eof occured) explicitly.
 *
 * Revision 1.33  2000/04/16 15:52:12  gellert
 * fixed some short lines (the end of them was left out while cut and
 * paste)...
 *
 * Revision 1.32  2000/04/16 15:27:40  gellert
 * output_string called reset_buffers() which could lead to negative string
 * lengths in calls to itself. Fixed and added some checks ;)
 *
 * Revision 1.31  2000/03/07 15:06:49  goldbach
 * added GPL info
 *
 * Revision 1.30  2000/03/02 16:00:31  goldbach
 * modified: get_next_tag(): if we're skipping parts of the document the search
 * for the next tag is modified to check that the following is a closing tag.
 * That way we're safely processing script code with arithmetic expressions.
 *
 * Revision 1.27  2000/02/23 17:45:22  gellert
 * If the end of a comment or tag is not in the inbuf, httpf
 * now memorizes, what is to be searched: The end of a comment
 * or the end of a tag. So the correct end of the comments
 * is now found even in this case.
 *
 * Revision 1.26  2000/02/23 15:25:32  gellert
 * Fixed a bug in the finding of the end of a comment. The test, if the
 * found end-of-comment was behind the end of the read buffer, failed.
 *
 * Revision 1.25  2000/02/16 17:07:27  goldbach
 * modified log messages (thread id was missing)
 *
 * Revision 1.24  2000/02/16 14:21:02  goldbach
 * removed function make_test_file() that created a file with just a big comment.
 *
 * Revision 1.23  2000/02/16 13:33:43  goldbach
 * replaced remaining calls of syslog >:)
 *
 * Revision 1.22  2000/02/01 17:02:22  gellert
 * Fixed a bug (if x >= y)  --> (if x == y)
 *
 * Revision 1.21  1999/08/19 13:39:35  gellert
 * small fix in the handling of comments (now we don't expect the
 * number of "-" at the end of a comment to be even).
 *
 * Revision 1.20  1999/08/05 14:39:47  goldbach
 * moved code from get_end_of_tag to extra function skip_comment().
 * Now httpf processes comments correctly. Hopefully :)
 *
 * Revision 1.19  1999/08/04 13:36:20  gellert
 * Just fixed a bug in the part that has to find the end of a comment.
 * This works now...
 *
 * Revision 1.18  1999/07/28 14:18:52  gellert
 * now the scanning of comments works correctly:
 * search for -- then find the next >
 *
 * Revision 1.17  1999/07/22 16:02:41  gellert
 * replaced strchr by strnchr
 *
 * Revision 1.16  1999/07/22 15:35:03  goldbach
 * yet again cleanup for logmsg()
 *
 * Revision 1.15  1999/07/22 14:36:02  gellert
 * improved tag scanning (find_end_of_tag), including nested quotes.
 *
 * Revision 1.14  1999/07/22 14:28:01  goldbach
 * minor cleanup for logmsg() (again)
 *
 * Revision 1.13  1999/07/16 12:49:05  goldbach
 * fixed 'bug' that mumbled about illegal inbuf offset.
 *
 * Revision 1.12  1999/07/16 09:48:49  goldbach
 * minor cleanup (warning)
 *
 * Revision 1.11  1999/07/15 20:11:10  gellert
 * removed a comment
 *
 * Revision 1.10  1999/07/15 19:59:48  gellert
 * get_end_of_tag was extended. it now handles comments
 * correctly: if the tag is a comment, the it searches
 * for the next "-->" otherwise it searches for the
 * next '>'.
 *
 * Revision 1.9  1999/07/14 09:50:04  goldbach
 * removed last strncpy.
 *
 * Revision 1.8  1999/06/30 13:25:44  goldbach
 * removed test dumps to stderr
 *
 * Revision 1.7  1999/06/29 12:20:01  gellert
 * just fiddled with the array boundaries to prevent
 * errors.
 *
 * Revision 1.6  1999/06/15 16:18:15  goldbach
 * bug: read_body_after_broken_tag() killed some contents always...
 *
 * Revision 1.5  1999/06/15 14:40:34  goldbach
 * if there are more than HTTP_BUFSIZE bytes between '<' and '>' this
 * so-called "tag" is thrown awaz silently. Well, okay... there's a log
 * message that tells you how many bytes are suppressed.
 *
 * This is quite handy when binaries are sent as text/html (like Apache
 * does it. Try this at home kids: give a binary the extension '.html'
 * and Apache will send it as text/html :(
 *
 * Revision 1.4  1999/06/09 20:33:53  gellert
 * new version. now the parsing of tags and attributes works.
 * and it still works if the client cancels the transmission
 * (stop button in browser).
 *
 * Revision 1.3  1999/05/27 16:47:59  goldbach
 * minor cleanup for logmsg()
 *
 * Revision 1.2  1999/05/27 11:29:35  gellert
 * well, now it does what we want. just small mistakes (eg end of
 * comments) are done.
 *
 * Revision 1.1  1999/05/12 16:07:11  goldbach
 * first version
 *
 *
 */


#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include "mythreads.h"
#include "misc.h"
#include "filter_html.h"
#include "parse_tag.h"
#include "parse_attributes.h"
#include "parse_tools.h"
#include "log.h"
#include "read.h"

/* prototypes */
static int parse(TCT*);
static int copy_pretag_contents(char *, TCT *);
static char *get_next_tag(TCT*);
static int read_body_after_broken_tag(TCT*);
static int check_kind_of_tag(TCT*);

/*
 * filter_html()
 *
 * Run over the inbuf and filter the HTML code we find.
 * After this function is called the output buffer is ready
 * for sending and the input buffer is ready for receiving.
 * Note: reset_buffers() may NOT be called afterwards --- buffer
 * offsets are set inside these functions.
 * Main loop: get start of next tag, copy everything between current position
 *   up to this next tag, handle the tag.
 *
 * Returns:
 * -1 -- an error occurred
 * other -- everything's normal
 */
int filter_html(TCT *tct) {    
  int result=OKAY;
  
  logmsg(LOG_INFO, "[%d] Filtering %d bytes of HTML code.",
	 tct->tid, tct->inlen-tct->inoffset); 

  tct->parse_state=PARSE_CONTENT;

  /* parse while everything's okay and we're in the buffer */
  while(result == OKAY && tct->inoffset<tct->inlen) {
    if (parse(tct)==FAILURE) {
      if (tct->error==NEED_MORE_INPUT) {
	/* read more data */
	if (!tct->more_input_data) {
	  logmsg(LOG_INFO, "[%d] No more data to read",
		 tct->tid);
	  break;
	}
	logmsg(LOG_INFO, "[%d] Refilling buffer in parse state %d",
	       tct->tid, tct->parse_state);
	result=handle_broken_tag(tct);
      } else {
	logmsg(LOG_INFO, "[%d] Error %d whilst filtering HTML",
	       tct->tid, tct->error);
	/* break on other errors */
	break;
      }
    }

    logmsg(LOG_DEBUG, "[%d] Processed %d of %d bytes in input buffer.",
	   tct->tid, tct->inoffset, tct->inlen);
  }

  /* send output buffer */
  if (send(tct->insocket, tct->outbuf, tct->outoffset, 0)<0) {
    logmsg(LOG_INFO, "[%d] Sending on client socket failed with errno %d.",
	   tct->tid, errno);
    tct->error=CLIENT_SOCKET;
  }
  reset_buffers(tct);
  return OKAY; /* processing completed */
}

int parse(TCT *tct) {
  if(tct->error!=NO_ERROR) return FAILURE;
  switch(tct->parse_state) {
  case PARSE_CONTENT:
    if (copy_pretag_contents(get_next_tag(tct),tct)==FAILURE)
      return FAILURE;
    tct->parse_state=PARSE_TAG;
    break;
  case PARSE_TAG:
    if(check_kind_of_tag(tct)==TAG_SPECIAL)
      break;

    if (parse_tag(tct)==FAILURE)
      return FAILURE;
    break;
  case PARSE_ATTRIBUTE:
    if (parse_attributes(tct)==FAILURE)
      return FAILURE;
    break;
  case PARSE_SKIP:
    if(skip_to_end_of_tag(tct)==FAILURE)
      return FAILURE;
    tct->parse_state=PARSE_CONTENT;
    break;
  case PARSE_COMMENT:
  case PARSE_COMMENT1:
    if (skip_comment(tct)==FAILURE)
      return FAILURE;
    break;
  }
  
  return OKAY;
}


/*
 * get_next_tag()
 *
 * Find next occurrence of < in inbuf+inoffset.
 */
static char *get_next_tag(TCT *tct) {
  char *tag_pos;
    
  /* check if the buffer is processed */
  if (tct->inoffset==tct->inlen)
    return NO_MORE_TAG;
  
  /* check if there's some error (we may have processed too much) */
  if (tct->inoffset>tct->inlen) {
    logmsg(LOG_ERR, "[%d] Illegal inbuf offset (%d of %d), aborting.", 
	   tct->tid, tct->inoffset, tct->inlen);
    return NO_MORE_TAG;
  }
  
  logmsg(LOG_DEBUG, "[%d] Inbuf offset is %d.", tct->tid, tct->inoffset);
  tag_pos = memchr(tct->inbuf+tct->inoffset, '<',
		   tct->inlen-tct->inoffset); /* find < */
  
  do {
	
	/*
	 * here we're using this implied logic (TODO: doc :))
	 * skipping is done between tags of *one* HTML element.
	 * FIXME: check config upon init when this works. If this
	 * code remains here we might as well use just *one* entry
	 * entry in the config file: "replskip" implies "replendskip..."
	 * This would also reduce the number of possible config. mistakes  ;)
	 *
	 * Skipping may be ended only when we find a "</" since
	 * it starts with an opening tag and ends with a closing tag.
	 * Thus, when finding a "<" we check if we're currently skipping
	 * contents. If so, we just keep on searching for the next tag introducer.
	 * The "<" just found is probably part of an arithmetic expression in script code...
	 */
	if (tag_pos==NULL) {
	  logmsg(LOG_DEBUG, "[%d] No closing '>' found.", tct->tid);
	  return NULL;
	}

	if (tct->skip_flag==1) { /* skipping? */
	  if ((tct->inoffset+1<tct->inlen) && (tag_pos[1]=='/')) { /* still in buf and is there "</"? */
		/* do return */
		return tag_pos;
	  } else {
		/* don't return, continue searching */
		tct->inoffset = tag_pos-tct->inbuf;
		tag_pos = memchr(tag_pos+1, '<',
						  tct->inlen-tct->inoffset); /* find < */
	
	  }
	} else {
	  /* just return the position of the new tag if we're not skipping */
	  return tag_pos;
	}
  } while (tag_pos);
  
  return tag_pos;
}

/*
 * check_kind_of_tag
 *
 * Performs a very rough check of the "tag":
 * !DOCTYPE is copied and comment is skipped. Both
 * set the tag_action attribute to TAG_SPECIAL.
 * Other tags are left untouched.
 *
 */
static int check_kind_of_tag(TCT *tct) {
  if (strncasecmp(tct->inbuf+tct->inoffset, "<!DOCTYPE",9)==0) {
    /* doctype: just copy it to output buffer */
    copy_doctype(tct->inbuf+tct->inoffset,tct);
    tct->parse_state = PARSE_CONTENT;
    return TAG_SPECIAL;
  } else if (strncmp(tct->inbuf+tct->inoffset, "<!--",4)==0) {
    /* comment? skip it! */
    logmsg(LOG_DEBUG, "[%d] Skipping comment", tct->tid);
    if(output_string("<!-- comment skipped -->",24,tct)==FAILURE) return FAILURE;

    tct->inoffset+=+3;	/* skip '!--'  */
    tct->parse_state=PARSE_COMMENT;
    return TAG_SPECIAL;
  } else {
    /* otherwise expect to have a normal tag */
    return TAG_PROCESS;
  }
}


/*
 * copy_pretag_contents
 *
 * Copy contents from inbuf to outbuf. This function is called
 * to do a verbatim copy between tags. It sets the error attribute
 * to NEED_MORE_INPUT if there's no tag in the buffer.
 */
static int copy_pretag_contents(char *tag, TCT *tct) {
  /* 
   * copy inbuf from the place we stopped working before
   * up to the tag. If we didn't find a tag, copy the rest of the
   * buffer.
   */

  if (tag!=NO_MORE_TAG) {
    /* if we have a tag in the rest of the buffer */
    output_string(tct->inbuf+tct->inoffset,
		  tag-(tct->inbuf+tct->inoffset),
		  tct);
    logmsg(LOG_DEBUG, "[%d] Copying %d bytes of content to output buffer.",
	   tct->tid, tag-(tct->inbuf+tct->inoffset));
    
    /* point at new tag */
    tct->inoffset=tag-tct->inbuf;
    return OKAY;    
  } else {
    logmsg(LOG_INFO, "[%d] No more tags in this buffer.", tct->tid);
    logmsg(LOG_DEBUG, "[%d] Am at offset %d.", tct->tid, tct->inoffset);
	
    if (tct->inlen-tct->inoffset>=0) {
      /* copy the complete rest of the buffer  */
      output_string(tct->inbuf+tct->inoffset,
		    tct->inlen-tct->inoffset,
		    tct);
      tct->inoffset=tct->inlen;
    } else {
      logmsg(LOG_INFO, "[%d] length of pretag contents is negative", tct->tid);
      logmsg(LOG_INFO, "     inlen=%d, inoffset=%d", tct->inlen, tct->inoffset);
    }
    tct->error=NEED_MORE_INPUT;
    return FAILURE;
  }
}


int handle_broken_tag(TCT *tct) {
  int len;

  logmsg(LOG_DEBUG, "[%d] Handling broken tag.", tct->tid);
  len=tct->inlen-tct->inoffset;
  logmsg(LOG_DEBUG, "[%d] Copying %d bytes from offset %d to the beginning of inbuf",
		 tct->tid, len, tct->inoffset);
  
  /* copy inbuf from tag till end to start of inbuf */
  memmove(tct->inbuf, tct->inbuf+tct->inoffset, len);
  
  reset_inbuf(tct); /* empty the input buffer */
  
  /* we already have a (small) chunk of valid data in the buffer */
  tct->inlen = len;

  /* read next block(s) */
  return read_body_after_broken_tag(tct);
    
  /*
   * now the part containing the broken tag is at the beginning
   * of the inbuf. it's vital *not* to call reset_buffers()
   * afterwards since we set the inlen attribute.
   */
}

/*
 * read_body_after_broken_tag
 *
 * Read body until we find a closing brace. If the amount of data between
 * the braces '<' and '>' exceeds the buffer size we silently throw the buffer
 * contents away.
 */
static int read_body_after_broken_tag(TCT *tct) {
    
  logmsg(LOG_DEBUG, "[%d] Entering read_body_after_broken_tag()", tct->tid);

  tct->error=NO_ERROR; /* reset error indicator */
  if(tct->more_input_data==YES) {
    logmsg(LOG_DEBUG, "[%d] Reading next server body", tct->tid);
    if ((read_full_server_block(tct)==FAILURE)) {
      /* leave if reading failed */
      tct->error=SERVER_SOCKET;
      logmsg(LOG_INFO, "[%d] Reading next block of data failed.",
	     tct->tid);
      return FAILURE;
    }
    return OKAY;
  } else { /* no more input to read */
    logmsg(LOG_INFO, "[%d] No more data to read.", tct->tid);
    return FAILURE;
  }
}

