/*
    Copyright (C) 2000 DFN-FWL (http://www.fwl.dfn.de/fwl)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/* httpf.c                                         og 09.02.99 */
/* was before: proxy.c
 * previous development see MGproxy, up to version 1.10
 *
 * $Id: httpf.c,v 1.30 2001/03/03 17:25:51 glauschwuffel Exp $
 * $Log: httpf.c,v $
 * Revision 1.30  2001/03/03 17:25:51  glauschwuffel
 * The check for binary data declared as text/html is now available on
 * Linux, too (protected unsafe rand() with semaphore).
 *
 * Expanded the error page when binary data is found a bit:
 * the contents surrounding the suspicious byte is now displayed.
 *
 * Revision 1.29  2001/03/01 15:56:59  glauschwuffel
 * cleanup for new modules: args.c, threads.c etc.
 *
 * Revision 1.28  2000/06/19 22:06:33  glauschwuffel
 * Major overhaul of the HTML parser. Now we have something like a
 * state machine (yes, I know that I should've done it that way
 * in the first place ;)).
 *
 * Split up some modules (misc.c, copy_pkts.c). filter_html.c
 * is much more cleaner now. I think that everyone should be able
 * to understand how the HTML parser works now. Especially
 * the skipping of comments is easy to get now ;)
 *
 * This seems to be getting cleaner...
 *
 * Revision 1.27  2000/05/07 15:19:37  glauschwuffel
 * added global var httpf_id which holds a nice HTML comment telling that the document was processed by our proxy. acdded inclusion of cfglists.h. changed inclusion of strings.h to string.h. made the solaris-only bit a local block whithin span_threads.
 *
 * Revision 1.26  2000/04/24 21:22:47  gellert
 * fiddled around with inet address. now client_allowed works on linux
 * and solaris.
 *
 * Revision 1.25  2000/04/24 19:44:34  gellert
 * Added new feature: clients entry in httpf.cfg. Only configured
 * clients are allowed to connect to httpf.
 * The handling of the lists of protocols and the lists
 * of allowed clients are now handled in cfglists.c
 *
 * Revision 1.24  2000/03/07 15:18:00  gellert
 * some changes to port to linux!!!
 *
 * Revision 1.23  2000/03/07 15:06:51  goldbach
 * added GPL info
 *
 * Revision 1.22  2000/03/01 13:33:17  gellert
 * Added a new commandline option "-n" (nodaemon), so the proxy stays
 * in foreground for debugging.
 *
 * Revision 1.21  2000/02/23 15:42:55  goldbach
 * fixed sem_init() call in httpf.c
 *
 * Revision 1.20  2000/02/23 15:09:26  goldbach
 * now we use POSIX-semaphores instead of Solaris-semaphores
 *
 * Revision 1.19  1999/11/26 10:47:07  goldbach
 * New actions introduced: replskip and replendskip skip sections of documents, i.e. data won\'t be sent to the client.
 *
 * Revision 1.18  1999/08/04 11:41:44  gellert
 * Changed the type check. Now there are two kinds of entries in httpf.cfg:
 * 1) ftype mimetype offset pattern: If the given mimetype matches, than
 *    it is checked whether the block contains pattern at offset.
 * 2) ftype mimetype: If the given mimetype matches, no further check follows.
 *    This makes sense for types like text/plain...
 * These changes required the change of the threadcontext, there is now a
 * character string containing the given mimetype of the server response.
 *
 * Revision 1.17  1999/07/28 14:20:07  gellert
 * some changes to the calls of logmsg
 *
 * Revision 1.16  1999/07/22 14:28:01  goldbach
 * minor cleanup for logmsg() (again)
 *
 * Revision 1.15  1999/07/22 13:24:16  goldbach
 * minor cleanup for logmsg()
 *
 * Revision 1.14  1999/07/14 09:50:04  goldbach
 * removed last strncpy.
 *
 * Revision 1.13  1999/07/14 08:42:04  gellert
 * inserted htons to convert port number to network byte order
 *
 * Revision 1.12  1999/06/29 12:21:30  gellert
 * now the debug level is set to (not incremented by) the given
 * command line argument.
 *
 * Revision 1.11  1999/06/23 16:59:53  gellert
 * now the socket option SO_REUSEADDR is set.
 * this prevents us from choosing a new port
 * after every test...
 *
 * Revision 1.10  1999/06/23 10:58:23  gellert
 * ok, inluded setting of reslimit
 *
 * Revision 1.9  1999/06/09 20:33:53  gellert
 * new version. now the parsing of tags and attributes works.
 * and it still works if the client cancels the transmission
 * (stop button in browser).
 *
 * Revision 1.8  1999/05/27 16:47:59  goldbach
 * minor cleanup for logmsg()
 *
 * Revision 1.7  1999/05/27 11:29:35  gellert
 * well, now it does what we want. just small mistakes (eg end of
 * comments) are done.
 *
 * Revision 1.6  1999/05/12 16:04:21  goldbach
 * major clean-up session
 *
 * Revision 1.5  1999/04/28 09:18:40  gellert
 * added the modules to read in the config file and place the
 * tags and attributes into a hash table.
 *
 * Revision 1.4  1999/03/02 11:28:08  goldbach
 * added buffer offsets initialization
 *
 * Revision 1.3  1999/02/16 17:44:34  gellert
 * some adaptions, eg new parameter -v etc.
 *
 * Revision 1.2  1999/02/09 13:39:30  gellert
 * removed all the "ifdef ssl/des3/arc4" - parts, so the code is now
 * a little bit simpler & shorter :-)
 *
 * Revision 1.1.1.1  1999/02/09 13:20:07  gellert
 * Import of httpf
 *
 */


#include <sys/param.h>
#include <sys/resource.h>


#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <syslog.h>
#include <errno.h>
#include <signal.h>
#include <pthread.h>
#include <semaphore.h> /* thread init */
#include "httpf.h"
#include "mythreads.h"
#include "misc.h"
#include "copy-pkts.h"
#include "open_conn.h"
#include "taghash.h"
#include "read_config.h"
#include "cfglists.h"
#include "log.h"
#include "threads.h"
#include "args.h"
#include "signal.h"

extern ARGS args;

int resource_flag = 0; 	/* flag for child: run setrlimit after forking */

char *httpf_id = "<!-- This page was frobnicated by httpf $Revision: 1.30 $.\n"
"See http://httpf.sourceforge.net for a description of this security proxy.\n"
"Status: EXPERIMENTAL -->\n\0";

/* random seed for threads */
unsigned int rand_r_seed;

#ifdef linux
sem_t rand_sema;
#endif

/* used in open_conn as well */
int setsocksize(int socket, int size) {
  if(setsockopt(socket, SOL_SOCKET, SO_SNDBUF,
		(char *)&size, sizeof(int)) < 0) {
    logmsg(LOG_ERR, "Setting SNDBUF for socket %d failed.", socket);
      return FAILURE;
  }
  
  if(setsockopt(socket, SOL_SOCKET, SO_RCVBUF,
		(char *)&size, sizeof(int)) < 0)  {
    logmsg(LOG_ERR, "Setting RCVBUF for socket %d failed.", socket);
    return FAILURE;
  }
  return OKAY;
}

static int setsockreuse( int socket) {
  int on=1;
  
  if(setsockopt(socket, SOL_SOCKET, SO_REUSEADDR,
		(char *) &on, sizeof (on)) < 0) {
    logmsg(LOG_ERR, "Setting REUSEADDR for socket %d failed.", socket);
    return FAILURE;
  }
  
  return OKAY;
}

static int open_tcp_socket() {
  int sockfd;
  
  /* Open a TCP socket (an Internet stream socket) */
  
  if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    logmsg(LOG_ERR, "Unable to open stream socket");
    exit(EXIT_FAILURE);
  }
  
  /* Socketsize fuer Clientseite fruehzeitig setzen. 
   * wird an sockets, die ueber accept kommen, vererbt.
   * nur so koennen window-sizes richtig ausgehandelt werden.
   */
  if (args.client_socket_size)
    setsocksize(sockfd, args.client_socket_size);
  
  setsockreuse(sockfd);

  return sockfd;
}

void bind_local_address(int socket) {
  struct sockaddr_in server_addr;
  
  /* bind our local address so that the client can send to us */
  bzero((char *) &server_addr, sizeof(server_addr));
  server_addr.sin_family      = AF_INET;
  server_addr.sin_addr.s_addr = htonl(INADDR_ANY);
  server_addr.sin_port        = htons(args.server_port);
  
  if (bind(socket, (struct sockaddr *) &server_addr, 
	   sizeof(struct sockaddr_in)) < 0) {
    logmsg(LOG_ERR, "Unable to bind local address (errno %d)", errno);
    exit(EXIT_FAILURE);
  }
}


int main (int argc, char *argv[]) {
  struct sockaddr_in	cli_addr;
  int clilen = sizeof(cli_addr);
  int sockfd;

  /* use current time as random seed */
  rand_r_seed = (unsigned int)time(NULL);

#ifdef linux
  /* rand() isn't thread safe and linux has no rand_r */
  sem_init(&rand_sema, 0, 1);
#endif

  initialize_logging();
  setup_signal_handler();
  set_default_args();
  process_args(argc,argv);

  /* doesn't work
     initialize_deadlock_checker();
     start_deadlock_checker();
  */

  logmsg(LOG_NOTICE, "Log facility is %d", args.log_facility);
  logmsg(LOG_INFO, "Socket sizes are: server %d, client %d, copy %d.",
	 args.server_socket_size, args.client_socket_size, args.copy_size);

  read_config(args.config_file);  

  if (args.daemon_flag==YES) {
    if (mkdaemon()) { 
      logmsg(LOG_ERR, "Unable to create daemon process"); 
      exit(EXIT_FAILURE); 
    } 
  }

  setsid();   
  /* leider setzt setsid() die limits zurueck */
  if (resource_flag) {
    set_resource_limit();
  }

/* close stdin, stdout, stderr if this should be a real daemon */
  close(0); close(1); close(2);
  
  /* Keine Zombies bitte */
#ifdef FREEBSD
  signal(SIGCHLD, SIG_IGN);
#else
  signal(SIGCLD, SIG_IGN);
#endif

  /* and no killing if we get SIGPIPE */
  /* this happens if we sent something to the client but he
     canceled the transfer (= stop button) */
  signal(SIGPIPE, SIG_IGN);
   
  initialize_worker_threads();

  sockfd = open_tcp_socket();
  bind_local_address(sockfd);

  if (listen(sockfd, 5) == -1) {
    logmsg(LOG_ERR, "Listen failed (errno %d)", errno);
    exit(1);
  }
  
  logmsg(LOG_NOTICE, "Starting at port %d", args.server_port);
  
  for ( ; ; ) {
    int newsockfd;

    /*
     * Wait for a connection from a client process.
     * This is an example of a concurrent server.
     */
    if ((newsockfd = accept(sockfd, (struct sockaddr *) &cli_addr,
			    &clilen)) == -1) {
      logmsg(LOG_ERR, "Unable to create socket for request connection (accept failed with errno %d)", errno);
      usleep (500); /* wait for 500 ms */
    } else { 
      /* new socket from kernel */      
      logmsg(LOG_NOTICE, "Created socket for request connection to %s:%d", 
	     inet_ntoa(cli_addr.sin_addr),
	     cli_addr.sin_port);

      if (!client_allowed(cli_addr.sin_addr.s_addr)) {
	logmsg(LOG_NOTICE, "Not allowing connect attempt from unknown client");
	close(newsockfd);
	goto accept_new_sock;
      }

      give_job_to_thread(args.number_of_threads, newsockfd);
    }

  accept_new_sock:
    /* solaris cc expects at least something here */
    ;
  }
}
