/*
    Copyright (C) 2000 DFN-FWL (http://www.fwl.dfn.de/fwl)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
 * $Id: misc.c,v 1.35 2001/03/01 15:59:58 glauschwuffel Exp $
 *
 * $Log: misc.c,v $
 * Revision 1.35  2001/03/01 15:59:58  glauschwuffel
 * usage is now in args.c
 *
 * Revision 1.34  2000/06/19 22:06:33  glauschwuffel
 * Major overhaul of the HTML parser. Now we have something like a
 * state machine (yes, I know that I should've done it that way
 * in the first place ;)).
 *
 * Split up some modules (misc.c, copy_pkts.c). filter_html.c
 * is much more cleaner now. I think that everyone should be able
 * to understand how the HTML parser works now. Especially
 * the skipping of comments is easy to get now ;)
 *
 * This seems to be getting cleaner...
 *
 * Revision 1.33  2000/05/08 13:11:30  glauschwuffel
 * another legal_constituent: +
 *
 * Revision 1.32  2000/05/07 15:26:27  glauschwuffel
 * changed inclusion of strings.h to string.h. moved function val_end to parse_tag.c. made function find_unquoted_end recognize the end of tags... new procession stream. function legal_constituent now allows more characters. new func legal_val_constituent.
 *
 * Revision 1.31  2000/04/24 18:00:37  gellert
 * cleaned up the return values to get rid of integer-pointer warnings.
 *
 * Revision 1.30  2000/04/16 15:32:55  gellert
 * output_string called reset_buffers() which could lead to negative string
 * lengths in calls to itself. Fixed and added some checks ;)
 *
 * Revision 1.29  2000/03/07 15:18:01  gellert
 * some changes to port to linux!!!
 *
 * Revision 1.28  2000/03/07 15:06:52  goldbach
 * added GPL info
 *
 * Revision 1.27  2000/03/01 13:33:16  gellert
 * Added a new commandline option "-n" (nodaemon), so the proxy stays
 * in foreground for debugging.
 *
 * Revision 1.26  2000/02/23 15:11:26  goldbach
 * now we use POSIX-semaphores instead of Solaris-semaphores
 *
 * Revision 1.25  2000/02/21 15:08:46  gellert
 * *** empty log message ***
 *
 * Revision 1.24  2000/02/21 14:56:19  gellert
 * Removed the unused function "output_data"
 *
 * Revision 1.23  1999/12/29 17:02:59  gellert
 * added a comma as a legal_constituent for tags and attributes without quotes.
 *
 * Revision 1.22  1999/11/30 16:57:53  gellert
 * skip flag: initialization moved to beginning of copy-pakets. read_header: empty_line is called with the length of the read block, not the full HTTP_BUFSIZE.
 *
 * Revision 1.21  1999/11/26 10:47:07  goldbach
 * New actions introduced: replskip and replendskip skip sections of documents, i.e. data won\'t be sent to the client.
 *
 * Revision 1.20  1999/11/18 12:56:26  gellert
 * fixed a bug in "logmsg". Now the message string is NOT interpreted
 * any more...  syslog(level, logbuf) --> syslog(level, "%s", logbuf)
 *
 * Revision 1.19  1999/11/11 15:51:49  gellert
 * Removed a "flush(stderr" command.
 * Added "_" as allowed char for an attribute.
 *
 * Revision 1.18  1999/08/19 13:43:30  gellert
 * some changes in the handling of quoted values for attributes
 * (now according to html documentation 3.2.2).
 * allowed '?' in unquoted values.
 *
 * Revision 1.17  1999/08/18 12:03:41  gellert
 * gregor fixed the nasty "xxx=''" bug. maybe now quotes really work.
 * removed an unused procedure (a_relevant).
 *
 * Revision 1.16  1999/08/09 13:27:41  goldbach
 * '#' and '%' are now legal constituents, too. this is due to the real world: '#' is used
 * in setting the bakground colour and '%' is used in widths etc. ...
 *
 * Revision 1.15  1999/08/09 12:04:33  goldbach
 * legal_constituent changed to accept '/' and ':' as chars in
 * values without surrounding quotes. This should be still safe.
 *
 * Revision 1.14  1999/07/28 14:22:22  gellert
 * some routines moved to here (legal_constituent etc).
 *
 * Revision 1.13  1999/07/22 14:30:54  gellert
 * some improvements of the determination of the tag end,
 * now nested quotes etc are supported.
 * changed config option replace to replabort/replcont.
 *
 * Revision 1.12  1999/07/16 09:47:55  goldbach
 * minor cleanup (warnings)
 *
 * Revision 1.11  1999/07/15 19:57:47  gellert
 * output_string now does a send if the string does not
 * fit into the outbuf.
 *
 * Revision 1.10  1999/07/14 09:50:05  goldbach
 * removed last strncpy.
 *
 * Revision 1.9  1999/06/30 12:51:33  goldbach
 * removed calls to logmsg() from strnchr
 *
 * Revision 1.8  1999/06/29 12:02:04  gellert
 * new logmsg procedure using vsnprintf.
 *
 * Revision 1.7  1999/06/23 16:58:16  gellert
 * logging is now always done to LOG_INFO, independent of the
 * level given to logmsg (level is just used to determine if it
 * should be logged at all).
 *
 * Revision 1.6  1999/06/23 09:11:01  gellert
 * added initialization of a variable to calm down purify.
 *
 * Revision 1.5  1999/06/09 20:33:54  gellert
 * new version. now the parsing of tags and attributes works.
 * and it still works if the client cancels the transmission
 * (stop button in browser).
 *
 * Revision 1.4  1999/05/27 16:44:15  goldbach
 * added logmsg()
 *
 * Revision 1.3  1999/05/27 11:29:36  gellert
 * well, now it does what we want. just small mistakes (eg end of
 * comments) are done.
 *
 * Revision 1.2  1999/05/12 16:04:21  goldbach
 * major clean-up session
 *
 * Revision 1.1  1999/05/04 09:18:51  goldbach
 * initial version. functions have been in modules where they don't belong.
 * 'tis much cleaner now :)
 *
 */

#include <stdio.h>
#include <strings.h>
#include <sys/types.h>
/* for mkdaemon */
#include <unistd.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>

#include <sys/resource.h>

/* and for the thread_context */
#include "mythreads.h"
#include "misc.h"

#include "httpf.h" /* HTTP_BUFZISE */
#include "log.h"   /* logmsg() */





/*
 * reset_buffers
 *
 * Call this before processing a new stream of data from either
 * the server or the client. Resets the buffers by setting
 * the buffer offsets and lengths to 0.
 */
void reset_buffers(TCT *tct) {
    tct->inoffset=0;
    tct->inlen=0;
    tct->outoffset=0;
    tct->outlen=0;
}

/*
 * reset_inbuf
 *
 * Resets the buffers by setting
 * the input buffer offset and length to 0.
 */
void reset_inbuf(TCT *tct) {
    tct->inoffset=0;
    tct->inlen=0;
}

/*
 * output_string
 *
 * Write string into output buffer and increase the according offset.
 * The output buffer will be sent to the insocket if the string
 * doesn't fit into the remaining space.
 * Returns:
 *   OKAY  -- string was written to output buffer. If a send
 *     was necessary it was done.
 *   FAILURE -- the string was too sizy for the output buffer
 *     (length > HTTP_BUFSIZE) of sending the output buffer failed.
 *     Sets error indicator. 
 * Bugs:
 *   If the string was too sizy for the output buffer it should be
 *   be sent to the insocket in chunks of HTTP_BUFSIZE.
 */
int output_string(char *s, size_t n, TCT *tct) {
  int restlen = 0;
  
  /* return immediately if we should skip contents */
  if (tct->skip_flag==1) return OKAY;
    
  /* return immediately if the string is too sizy for the buffer */
  /* FIXME: output the string in chunks of HTTP_BUFSIZE */
  if (n > HTTP_BUFSIZE) {
    tct->error=PARAMETERS;
    return FAILURE;
  }

  if (n<0) {
    logmsg(LOG_ERR, "[%d] BUG: Negative size in call of output_string()", tct->tid);
    tct->error=PARAMETERS;
    return FAILURE;
  }

  if (s==NULL) {
    logmsg(LOG_ERR, "[%d] BUG: Null pointer passed in call of output_string()", tct->tid);
    tct->error=PARAMETERS;
    return FAILURE;
  }

  restlen = HTTP_BUFSIZE - tct->outoffset;

  if (n > restlen) {
    /* this does not fit into the buffer */
	
    logmsg(LOG_DEBUG, "[%d] Output buffer full, send needed", tct->tid);
    /* send the outbuf so far */
    if (send(tct->insocket, tct->outbuf, tct->outlen, 0)<0) {
      logmsg(LOG_INFO, "[%d] Sending the output buffer failed with errno %d",
	     tct->tid, errno);
      tct->error=CLIENT_SOCKET;
      return FAILURE;
    }
	
    logmsg(LOG_DEBUG, "after send()");
    
    tct->outoffset = 0; tct->outlen = 0;
  }
    
  memcpy(tct->outbuf+tct->outoffset, s, n);
  tct->outoffset+=n; tct->outlen=tct->outoffset;

  tct->error=NO_ERROR;
  return OKAY;
}

/*
 * error_string
 *
 * Write string into error buffer and increase the according offset.
 */
void error_string(char *s, int n, TCT *tct) 
{
    memcpy(tct->errbuf+tct->erroffset, s, n);
    tct->erroffset+=n;
}


/*
 * find_eol
 *
 * Find end of line in any http buffer. This should be
 * \r\n... however, some people implement only \n or \r
 * so we have to accept these, too. The following character (-combos)
 * are an EOL: \n, \n\r, \r, \r\n (thus \n\n is still *two*
 * newlines).
 * Inputs:
 *    size -- #of bytes to search in
 *    replace -- if this true we replace both cr and lf
 *           with the null character.
 * Returns: 0 if no eol was found or end of line address.
 * Next character will be the new line.
 */
char *find_eol(char *buf, int size, int replace)
{
    char *cur;
 
    for (cur=buf; cur<buf+size; cur++) {
	/* is it \n or \n\r? */
	if (cur[0]=='\n') {
	    if (replace) cur[0]=0;
	    
	    if (cur[1]=='\r') {
		if (replace) cur[1]=0;
		return cur+1;
	    } else
		return cur;
	}

	
	/* is it \r or \r\n? */
	if (cur[0]=='\r') {
	    if (replace) cur[0]=0;
	    
	    if (cur[1]=='\n') {
		if (replace) cur[1]=0;
		return cur+1;
	    } else
		return cur;
	}
    }
    return 0;
    
}


/* Vom steuernden Terminal unabhaengig machen */
/* von Benecke uebernommen...
 */
int mkdaemon (void)
{
  int pid = fork();
  if (pid > 0)
    exit (0); /* Vaterprozess soll terminieren */
  if (pid == 0)
    {
      setsid();   
      
      chdir ("/"); /* in das Root-Verzeichnis wechseln, so dass wir
                      nicht durch ein UMOUNT der Partition von der wir
                      gestartet wurden, terniniert werden.
                      */
      umask (0);
    }
  return pid;
}





/* #include <stdio.h>
 * #include <sys/types.h>
 */
 
#define		MAXLINE			80

/* gg 29.12.99:
 * move these global variables below into the thread context...
 * These functions are unused for now :)
 */
			
u_long	lAddress=0;
int		wSpace=0;

void htoa(register char bHex, register char *pAsciiRet) {

	if (bHex >= 0 && bHex <= 9) *(pAsciiRet) = bHex + '0';
	else *(pAsciiRet) = (bHex - 10) + 'A';
}

void ctoa(register u_char bChar, register char *pAsciiRet) {
	htoa((bChar>>4),pAsciiRet++);
	htoa((bChar&0x0f),pAsciiRet);
}

void ltoa(register u_long lI, register char *pAsciiRet) {
	char	rgTemp[8];
	register int	I=0;
	register int	J;
	
	while ((lI>>4) != 0) {
		rgTemp[I++] = lI & 0x0f;
		lI = lI>>4;
	}
	rgTemp[I++] = lI;
	for (J=I; J<8; J++) rgTemp[J] = 0;
	while (J > 0) {
		J--;
		htoa(rgTemp[J], pAsciiRet++);
	}
}

void xdline(register char *pLine, register int cLine, FILE *logfd) {
	register int	I, J;
	register int	iCol;
	register int	iAlphaCol;
	char		pAddress[9];
	char		pData[3];
	u_char		Line[MAXLINE];

	for (iCol=0; iCol<MAXLINE; iCol++) Line[iCol] = ' ';
	iCol = 0;
	iAlphaCol = 60;
	ltoa(lAddress,pAddress);
	pAddress[8] = '\0';
	strcpy((char *) &Line[iCol],pAddress);
	iCol += 8;
	Line[iCol++] = ' ';
	Line[iCol++] = ' ';
	lAddress += 0x10;
	for (I=0; I<cLine; I++) {
		if ((*pLine >= ' ') && (*pLine <= '~'))
			Line[iAlphaCol++] = *pLine;
		else Line[iAlphaCol++] = '.';

		ctoa(*(pLine++),pData);
		pData[2] = '\0';
		strcpy((char *) &Line[iCol],pData);
		iCol += 2;
		Line[iCol++] = ' ';
	}
	Line[iAlphaCol++] = '\n';
	for (J=0; J<wSpace; J++)
		Line[iAlphaCol++] = '\n';
	Line[iAlphaCol] = '\0';

	/*
	for (I=0; I<iAlphaCol; I++)
		putchar(Line[I]);
	*/
	fprintf(logfd, "%s", Line);
}

void xdbuf(register char *pBuf, register int cBuf, FILE *logfd) {
	register int	nLine;
	register int	i;

	lAddress = 0;
	nLine = cBuf>>4;
	for (i=0; i<nLine; i++) {
		xdline(pBuf, 0x10, logfd);
		pBuf += 0x10;
	}
	i = cBuf & 0x0f;
	if (i > 0) {
		xdline(pBuf, i, logfd);
	}
}

	 

int test_write(int the_socket) {

  int 	nfds = getdtablesize(); /* select ...? */
  fd_set 	fds;	/* fuer select: liste mit sockin + sockout */
  static struct timeval	tout;
  
  FD_ZERO(&fds);
  
      tout.tv_sec = 60*60*2;
      tout.tv_usec = 0;

      FD_SET(the_socket, &fds);

      if (select(nfds, NULL, &fds, NULL, &tout) > 0) {
	  /* writeable? */
	  if (FD_ISSET(the_socket, &fds)) {
	      /* fprintf(stderr, "test_write: yes!\n"); */
	      return 1;
	  } else {
	      /* fprintf(stderr, "test_write: no!\n"); */
	      return 0;
	  }
      }
      
      /* fprintf(stderr, "test_write: no!\n"); */
      return 0;
}




