/*
    Copyright (C) 2000 DFN-FWL (http://www.fwl.dfn.de/fwl)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 * $Id: mythreads.h,v 1.18 2001/03/01 16:18:01 glauschwuffel Exp $
 *
 * $Log: mythreads.h,v $
 * Revision 1.18  2001/03/01 16:18:01  glauschwuffel
 * error constants redefinition, added context attributes
 *
 * Revision 1.17  2000/09/06 13:20:36  glauschwuffel
 * changed error constants. some values were not unique.
 *
 * Revision 1.16  2000/06/21 18:51:01  glauschwuffel
 * New error constants for proto_allowed().
 *
 * Revision 1.15  2000/06/19 22:06:33  glauschwuffel
 * Major overhaul of the HTML parser. Now we have something like a
 * state machine (yes, I know that I should've done it that way
 * in the first place ;)).
 *
 * Split up some modules (misc.c, copy_pkts.c). filter_html.c
 * is much more cleaner now. I think that everyone should be able
 * to understand how the HTML parser works now. Especially
 * the skipping of comments is easy to get now ;)
 *
 * This seems to be getting cleaner...
 *
 * Revision 1.14  2000/05/07 16:10:27  glauschwuffel
 * added babbling constants. expanded context: more_input_data is YES if the server has more data for us. last_valid_tag, last_valid_attr and last_valid_value hold pointers to last correctly parsed locations in input buffer
 *
 * Revision 1.13  2000/03/07 15:06:53  goldbach
 * added GPL info
 *
 * Revision 1.12  2000/02/23 17:45:22  gellert
 * If the end of a comment or tag is not in the inbuf, httpf
 * now memorizes, what is to be searched: The end of a comment
 * or the end of a tag. So the correct end of the comments
 * is now found even in this case.
 *
 * Revision 1.11  1999/12/01 17:53:02  gellert
 * bug-fix: field_name was a static pointer (used by all the threads!). ARGH!
 *
 * Revision 1.10  1999/11/26 10:47:08  goldbach
 * New actions introduced: replskip and replendskip skip sections of documents, i.e. data won\'t be sent to the client.
 *
 * Revision 1.9  1999/08/04 11:41:45  gellert
 * Changed the type check. Now there are two kinds of entries in httpf.cfg:
 * 1) ftype mimetype offset pattern: If the given mimetype matches, than
 *    it is checked whether the block contains pattern at offset.
 * 2) ftype mimetype: If the given mimetype matches, no further check follows.
 *    This makes sense for types like text/plain...
 * These changes required the change of the threadcontext, there is now a
 * character string containing the given mimetype of the server response.
 *
 * Revision 1.8  1999/05/12 16:04:23  goldbach
 * major clean-up session
 *
 * Revision 1.7  1999/03/30 12:30:07  goldbach
 * 1st surfable version
 *
 * Revision 1.6  1999/03/02 11:39:08  goldbach
 * added length information for buffers.
 *
 * Revision 1.5  1999/02/25 13:37:21  goldbach
 * added fields from the request line
 *
 * Revision 1.4  1999/02/18 16:46:48  goldbach
 * added loglevel
 *
 * Revision 1.3  1999/02/11 15:08:37  goldbach
 * Sockets fr aktuelle Verbindung eingefgt
 *
 * Revision 1.2  1999/02/11 14:30:55  goldbach
 * Puffer fuer Ein-, Ausgabe und Fehlerverarbeitung sowie Offsets
 * hinzugefuegt.
 *
 */

#ifndef __MYTHREADS__
#define __MYTHREADS__
#define WORKING 1
#define ERROR -1
#define READY 0
#define SCHEDULED 2

/* length of the reason phrase */
#define LEN_REASON 40

/* what are we searching for? the end of a tag or the end of a comment? */
#define	END_OF_TAG 0
#define	END_OF_COMMENT 1

/* babbling constants for misc use, e.g. for more_input_data */
#define YES 1
#define NO  0

#define OKAY 0
#define FAILURE -1

/* valid values of parse_state */
#define PARSE_TAG 1        /* we're parsing the tag */
#define PARSE_ATTRIBUTE 2  /* we're parsing an attribute */
#define PARSE_CONTENT 3    /* we're parsing content */
#define PARSE_SKIP 4       /* skip this tag */
#define PARSE_COMMENT 5    /* find -- */
#define PARSE_COMMENT1 6   /* encountered -- */
#define PARSE_COPY 7       /* skip this tag and copy what you skip (doctype) */

/* valid values of error */
#define NO_ERROR 0
#define END_OF_BUFFER 1   /* end of buffer */
#define BUFFER_OVERRUN 2  /* pointer>tct->inbuf+tct->inlen */
#define BUFFER_UNDERRUN 3 /* pointer<tct->inbuf */
#define NULL_POINTER 4    /* pointer=0 */
#define UNALLOWED_CHAR 5  /* this character is no allowed here */
#define UNALLOWED_PROTOCOL 6  /* this protocol is no allowed */
#define NO_PROTOCOLS_CONFIGURED 7  /* there are no protocols configured*/

#define NEED_MORE_INPUT 8   /* we should read more data from server */
#define UNKNOWN_ACTION 9    /* hm. the last action was unknown */
#define SERVER_SOCKET 10    /* error on server socket */
#define CLIENT_SOCKET 11    /* error on client socket */
#define SOCKET 12           /* error on socket */
#define PARAMETERS   13     /* parameters are wrong */
#define NO_END_OF_HEADER 14 /* the HTTP-Header does not end */

/* valid values of tag_action */
#define TAG_SKIP 1       /* skip it */
#define TAG_PROCESS 2    /* process it */
#define TAG_UNALLOWED 3  /* not allowed */
#define TAG_SPECIAL 4    /* 'special' tag like !doctype or comment */

#include <semaphore.h>
#include <sys/time.h>

typedef struct thread_context {
  int tid; /* eigene Thread ID, != pthread_self() !!!! */
  sem_t wait;
  int stat; /* <0 = error; == 0 = ready; > 0 = working */
  int sockfd;
  /* ab hier aufgabenspezifischer Krams */
  int ServerSockSize; 
  int ClientSockSize;
  int CopySize;
  /*
   * buffers for input, output and error handling.
   * NB: inbuf will be modified (e.g. insert null character
   * for CR, LF).
   */
  char *inbuf;
  char *outbuf;
  char *errbuf;
  /* buffer offsets. incremented whilst processing. */
  int inoffset;
  int outoffset;
  int erroffset;
  /* valid len */
  int inlen; /* bytes read */
  int outlen; /* bytes to be written */
  int errlen; /* bytes to be written */
  /* sockets for this connection */
  int insocket;
  int outsocket;
  /* log level */
  int loglevel;
  
  /* request line: http method */
  int method;
  /* request line: HTTP/major.minor */
  int version_major;
  int version_minor;
  /*
   * request line: URI
   * this will be somewhere at the start of inbuf
   */
  char *uri;
  /* response line: status */
  int response_status;
  /* response_line: reason phrase */
  char reason[LEN_REASON];
  /* MIME type id from server response (subrange) */
  char *mime_type;
  /* field_name: points to the actual fieldname in the http header
	 (used by copy_headerline in analyze_header.c) */
  char *field_name;
  /* flag for skipping parts of documents */
  int skip_flag;
  /* is there more input data from the server?
   * This will be set to 0 if the server sent an EOF.
   * 1 otherwise
   */
  int more_input_data;
  /*
   * These point to the items that have been parsed
   * correctly
   */
  char *last_valid_tag;
  char *last_valid_attr;
  char *last_valid_value;
  /* this tells which phase of processing we're in */
  int parse_state;
  /* if there's an error this will be set */
  int error;
  /* what do we do with tag? */
  int tag_action;
  /* used for skipping over tags...*/
  int angle_count, quote_count;
  /*
   * time before the last recv(). Used in
   * controlthread.c to see if there's a deadlock.
   */
  time_t recv_time;
  /* set to YES if the initial thread received the hangup signal */
  int hangup_flag;
  /* indicator that tells us if we the server response had a header */
  int response_has_http_header;
} TCT;


#endif


