/*
    Copyright (C) 2000 DFN-FWL (http://www.fwl.dfn.de/fwl)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/* 
 * $Log: open_conn.c,v $
 * Revision 1.8  2000/06/19 21:44:15  glauschwuffel
 * includes log.h instead of misc.h
 *
 * Revision 1.7  2000/03/07 15:06:53  goldbach
 * added GPL info
 *
 * Revision 1.6  1999/07/22 14:28:02  goldbach
 * minor cleanup for logmsg() (again)
 *
 * Revision 1.5  1999/07/14 09:50:06  goldbach
 * removed last strncpy.
 *
 * Revision 1.4  1999/06/09 20:47:42  gellert
 * just some little includes and prototypes against the
 * warnings.
 *
 * Revision 1.3  1999/05/27 16:48:00  goldbach
 * minor cleanup for logmsg()
 *
 * Revision 1.2  1999/02/09 13:39:30  gellert
 * removed all the "ifdef ssl/des3/arc4" - parts, so the code is now
 * a little bit simpler & shorter :-)
 *
 * Revision 1.1.1.1  1999/02/09 13:20:07  gellert
 * Import of httpf
 *
 */

#include <sys/socket.h>
#include <arpa/inet.h>
#include <syslog.h>
#include <errno.h>
#include "mythreads.h"
#include "httpf.h" /* setsocksize */
#include "log.h"

/* open_conn liefert Socket-FD auf offene Verbindung zu Ziel-Host
 * dest_addr  enthaelt Ziel-Host und Ziel-Port
 */

int
open_conn(dest_addr, SocketSize, tct)
    struct sockaddr_in	*dest_addr;
    int SocketSize;
    TCT *tct;
{
int sockfd;

  if ( ( sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    logmsg(LOG_ERR,"[%d] Error opening stream socket with errno %d.",
	   tct->tid, errno);
    return(-1);
  }

  /* Wenn SocketSize als Commandline-Optionen gesetzt, 
   * dann hier setzen. Sonst wird Systemdefault belassen.
   * Bei Fehlern gibt es Syslog-Meldung, es wird aber weitergemacht.
   */
  if (SocketSize)
    setsocksize(sockfd, SocketSize);

  logmsg(LOG_INFO,"[%d] Connect to %s:%d on socket %d.",
	 tct->tid,
	 inet_ntoa(dest_addr->sin_addr),
	 dest_addr->sin_port,
	 sockfd);

  if (connect(sockfd, (struct sockaddr *) dest_addr, sizeof(struct sockaddr) ) < 0) {
    logmsg(LOG_ERR,"[%d] Connect failed with errno %d.", tct->tid, errno);
    return(-1);
  }

  return sockfd;
}


