/*
    Copyright (C) 2000 DFN-FWL (http://www.fwl.dfn.de/fwl)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 * $Log: parse_tag.c,v $
 * Revision 1.30  2000/06/22 09:35:08  glauschwuffel
 * put back init of tct...count before changed to SKIP state
 *
 * Revision 1.29  2000/06/21 19:00:03  glauschwuffel
 * small patch for the test suite and a minor cleanup of log
 * messages
 *
 * Revision 1.28  2000/06/19 22:06:33  glauschwuffel
 * Major overhaul of the HTML parser. Now we have something like a
 * state machine (yes, I know that I should've done it that way
 * in the first place ;)).
 *
 * Split up some modules (misc.c, copy_pkts.c). filter_html.c
 * is much more cleaner now. I think that everyone should be able
 * to understand how the HTML parser works now. Especially
 * the skipping of comments is easy to get now ;)
 *
 * This seems to be getting cleaner...
 *
 * Revision 1.27  2000/05/09 14:35:15  gellert
 * after a illegal attribut go on with the next attribut (after the
 * next whitespace).
 *
 * Revision 1.26  2000/05/08 13:13:16  glauschwuffel
 * minor fixes for attribute processing
 *
 * Revision 1.25  2000/05/07 16:13:42  glauschwuffel
 * made functions local. now we process documents this way: parse each item bit by bit. we do not know the end of a tag anymore when entering parse_tag. this is the first step to be robust against unmatched quotes.
 *
 * Revision 1.24  2000/04/24 19:32:54  gellert
 * cleaned up the return values to get rid of integer-pointer warnings.
 *
 * Revision 1.23  2000/04/18 15:11:36  gellert
 * unallowed_href returned -1 if the attribute had no value, so
 * attributes without values would not be allowed. fixed.
 *
 * Revision 1.22  2000/04/16 15:38:58  gellert
 * fixed two bugs:
 * a) check_for_script was not called. and now it checks all the
 *    ampersands for a following brace, not just the first one.
 * b) we have to call proto_allowed not only for HREF, but also
 *    for the SRC attribute.
 *
 * Revision 1.21  2000/03/07 15:06:54  goldbach
 * added GPL info
 *
 * Revision 1.20  2000/02/21 14:11:52  gellert
 * And we shouldn't call logmsg with NULL-pointers... ;-)
 *
 * Revision 1.19  2000/02/21 13:57:50  gellert
 * In "unallowed_href": inserted a test, if the given value is
 * a NULL-pointer. Otherwise httpf segfaults at <A HREF>...
 *
 * Revision 1.18  2000/02/21 13:42:47  gellert
 * In the valeu of a HREF we now search not only for ":", but
 * also for &#58, to prevent the values "javascript&#58"
 *
 * Revision 1.17  1999/11/26 10:47:08  goldbach
 * New actions introduced: replskip and replendskip skip sections of documents, i.e. data won\'t be sent to the client.
 *
 * Revision 1.16  1999/11/12 12:16:20  gellert
 * Fixed:	If an attribute was logged only (and not replaced or
 * 	allowed), the attributes were processed afterwards
 * 	(so the "<" and the tag were ommitted, but the attributes
 * 	and the ">" were copied to the outbuf).
 *
 * Revision 1.15  1999/11/11 15:58:02  gellert
 * Removed declarations of some nonexistent functions.
 * Some new logmessages inserted.
 *
 * Revision 1.14  1999/08/09 12:09:32  goldbach
 * Changed unallowed_href to accept hrefs without *any* protocols at all :)
 * Bug: do_attr aborted after processing an attribute without a value. This
 * buggy behaviour must have been around since the removal of the high-level
 * strncpy() for everything between the angles... tables look fine now :)
 *
 * Revision 1.13  1999/07/28 14:43:05  gellert
 * moved some functions to misc.c
 *
 * Revision 1.12  1999/07/22 15:35:04  goldbach
 * yet again cleanup for logmsg()
 *
 * Revision 1.11  1999/07/22 14:30:52  gellert
 * some improvements of the determination of the tag end,
 * now nested quotes etc are supported.
 * changed config option replace to replabort/replcont.
 *
 * Revision 1.10  1999/07/15 17:01:22  gellert
 * added the check of href tags for allowed protocols.
 *
 * Revision 1.9  1999/07/14 09:50:06  goldbach
 * removed last strncpy.
 *
 * Revision 1.8  1999/06/30 14:10:15  goldbach
 * removed subtle bug in process attr
 *
 * Revision 1.7  1999/06/29 11:56:41  gellert
 * parsing changed: now there are 0-bytes written into the
 * input buffer to terminate the strings. so they need not
 * to be copied into some extra buffer...
 *
 * Revision 1.6  1999/06/23 11:14:40  gellert
 * deleted some never reached line
 *
 * Revision 1.5  1999/06/23 10:45:20  gellert
 * better checking of the tag,
 * bug fixes (segfaults).
 *
 *
 * $Id: parse_tag.c,v 1.30 2000/06/22 09:35:08 glauschwuffel Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>
#include <ctype.h>
#include "mythreads.h"
#include "taghash.h"
#include "cfglists.h"
#include "misc.h"
#include "parse_tag.h"
#include "parse_attributes.h"
#include "parse_tools.h"
#include "log.h"

int parse_tag(TCT*);
static int do_tag(TCT*);
static int process_tag(char*,TCT*);

static char *find_tagstart(char*,TCT*);
static char *find_tagend(char*,TCT*);

/*
 * parse_tag
 *
 * Parse the tag in the buffer.
 * Process the tag itself (do_tag()), check for attributes and
 * process them one by one if there are any (do_attr()).
 * Returns FAILURE on error, OKAY otherwise.
 * The error indicator is set to NEED_MORE_INPUT if there's
 * any other error than UNALLOWED_CHAR: when there's a problem
 * with the input buffer we'll read more data and try it again.
 */
int parse_tag(TCT *tct) {
  tct->inoffset++; /* skip the opening '<' */

  /* this is a normal tag, process it */   
  if (do_tag(tct)==FAILURE) {
    tct->inoffset--; /* step back to the '<' */

    switch(tct->error) {
    case UNALLOWED_CHAR: /* skip this tag, there were bad characters */
      logmsg(LOG_INFO,"[%d] Skipping malformed tag",
			tct->tid);
      if(output_string("<!-- malformed tag skipped -->",
		       30, tct)==FAILURE) return FAILURE;
      tct->inoffset++; /* skip the opening '<' */
      /* we're in a tag */
      tct->angle_count=1; tct->quote_count=0;
      tct->parse_state = PARSE_SKIP;
      return OKAY;
    case BUFFER_OVERRUN: /* we need more input data */
      tct->error=NEED_MORE_INPUT;
      return FAILURE;
    case CLIENT_SOCKET: /* user might have pressed 'Stop' */
      return FAILURE;
    case NEED_MORE_INPUT:
    default:
      logmsg(LOG_INFO,"[%d] Tag parsing needs more input data",
	     tct->tid,tct->error);
      tct->error=NEED_MORE_INPUT;
      return FAILURE;
    } /* switch */
  }

  /* no error */
  switch(tct->tag_action) {
  case TAG_UNALLOWED:
    if(output_string("<!-- unknown tag skipped -->",
		     28, tct)==FAILURE) return FAILURE;
    /* we're in a tag */
    tct->angle_count=1; tct->quote_count=0;
    tct->parse_state = PARSE_SKIP;
    break;
  case TAG_SKIP: /* skip it quietly */
    /* we're in a tag */
    tct->angle_count=1; tct->quote_count=0;
    tct->parse_state = PARSE_SKIP;
    break;
  case TAG_PROCESS: {
    char *ws;

    /* skip possible WS between tag and closing '>' */
    ws = skip_leading_whitespace(tct->inbuf+tct->inoffset,tct);    
    if (tct->error!=NO_ERROR) {
      tct->error=NEED_MORE_INPUT;
      return FAILURE;
    } else {
      /* copy the WS. remove this if the resulting
	 document ought to look cleaner */
      if(output_string(tct->inbuf+tct->inoffset,
		       ws-(tct->inbuf+tct->inoffset),tct)==FAILURE) return FAILURE;
    }
    
    tct->inoffset=ws-tct->inbuf;
    /* check if there are any attrs to process */
    if (tct->inbuf[tct->inoffset]=='>') {     /* no */
      logmsg(LOG_INFO, "[%d] This tag has no attributes.", tct->tid);
      tct->inoffset++; /* skip the closing '>' */
      if(output_string(">", 1, tct)==FAILURE) return FAILURE; /* close the tag */
      tct->parse_state=PARSE_CONTENT;
    } else { /* there are attributes */
      tct->parse_state=PARSE_ATTRIBUTE;
    }
    break;
  } /* case */
  } /* switch */
  return OKAY;
}

/*
 * do_tag
 *
 * Try to get and handle a tag in string. This is basically the same as
 * do_attr() except that a tag is slightly more than an attribute
 * (characterwise) -- the slash ('/') is a legal character, too.
 * See do_attr() for a description and notes.
 * Returns:
 *  PARSE_OKAY -- everything's normal, continue with attributes.
 *  PARSE_ERROR -- an error occurred, e.g. there were illegal charaters. check tct->error.
 */
static int do_tag(TCT *tct) {
  char *tag_start, *tag_end;
  char replaced;

  tct->parse_state = PARSE_TAG;
  tct->tag_action = TAG_UNALLOWED;

  logmsg(LOG_DEBUG, "[%d] Parsing tag from offset %d.",
	 tct->tid, tct->inoffset);

  tag_start = find_tagstart(tct->inbuf+tct->inoffset,tct);
  if (tag_start==NULL) return FAILURE;  

  tag_end = find_tagend(tag_start+1,tct);
  if (tag_end==NULL) return FAILURE;
  
  /* terminate tag. remember the char we replace */
  replaced=tag_end[0]; tag_end[0]=0;

  if (process_tag(tag_start, tct)==FAILURE) return FAILURE;

  tag_end[0]=replaced; /* put back */
  
  tct->inoffset=tag_end-tct->inbuf;   /* adjust inoffset */
  logmsg(LOG_DEBUG, "[%d] Adjusted inoffset to %d (tag end).",
		 tct->tid, tct->inoffset);

  return OKAY;
}



/*
 * process_tag
 *
 * Takes buffer containing the tag and the context.
 * Look up the tag in hash-table built from the configuration and
 * act like the action says.
 * Input: terminated tag and thread context
 * Returns:
 *  0  -- continue with the attributes
 * -1  -- tag not allowed, skip rest of tag string
 */
static int process_tag(char *tagbuf, TCT *tct) {
  char *replacement; /* from hash-table */
  int action; /* from hash-table */
  
  logmsg(LOG_INFO, "[%d] Processing tag %s",
	       tct->tid, tagbuf);
    
  /* find out what we are supposed to do */
  action = tag_action(tagbuf);
    
  if (action < 0) {
    /* not specified in configuration file */
    logmsg(LOG_NOTICE, "[%d] Not allowing tag %s", tct->tid, tagbuf);
    tct->tag_action = TAG_UNALLOWED;
    return OKAY;
  }
    
  if (action & (ACT_REPLABORT | ACT_REPLCONT | ACT_REPLSKIP |
		ACT_REPLENDSKIP)) {
    /* get replacement for this tag*/
    replacement=tag_replacement(tagbuf);
    if (action & ACT_LOG) {
      logmsg(LOG_NOTICE, "[%d] Replacing tag %s with '%s'",
	     tct->tid, tagbuf, replacement);
    }
	
    output_string("<", 1, tct); /* (1) */
    if(output_string(replacement, strlen(replacement),tct)==FAILURE) return FAILURE;
		
    if (action & ACT_REPLSKIP) {
      output_string(">", 1, tct);
      tct->skip_flag=1; /* skip the following contents */
      tct->tag_action = TAG_SKIP;
    } else if (action & ACT_REPLENDSKIP) {
      /* turn off skipping mode and write the replacement */
      tct->skip_flag=0;
      output_string("<", 1, tct); /* (2) */
      if(output_string(replacement, strlen(replacement),tct)==FAILURE) return FAILURE;
      output_string(">", 1, tct);
	    
      tct->tag_action = TAG_SKIP;
    } else if (action & ACT_REPLABORT) {
      output_string(">", 1, tct);
      tct->tag_action = TAG_SKIP;
    } else {
      tct->tag_action = TAG_PROCESS;
    }
  } else if (action & ACT_ALLOW) {
    if (action & ACT_LOG) {
      logmsg(LOG_NOTICE, "[%d] Allowing tag %s", tct->tid, tagbuf);
    }
    
    /* copy tag 'as is' to output buffer */
    output_string("<", 1, tct);
    if(output_string(tagbuf, strlen(tagbuf), tct)==FAILURE) return FAILURE;
    tct->tag_action = TAG_PROCESS;
  } else if (action & ACT_LOG) {
    /* just log it */
    logmsg(LOG_NOTICE, "[%d] Skipping tag %s", tct->tid, tagbuf);
    tct->tag_action = TAG_SKIP;
  } else {
    /* we should never get here */
    logmsg(LOG_ERR, "[%d] Unknown action %d not processed, skipping tag",
	   tct->tid, action);
    tct->tag_action = TAG_SKIP;
    return FAILURE;
  }
  return OKAY;
}


/*
 * find_tagstart
 *
 * Find start of tag.
 * See find_attrstart, the only difference is that we allow
 * a '/' at the start. 
 */
static char *find_tagstart(char *buf, TCT *tct) {
  char *tag_start;  

  /* skip whitespace */
  tag_start = skip_leading_whitespace(buf,tct);
  if(tct->error!=NO_ERROR) return NULL;

  if(legal_constituent(tag_start[0]) || tag_start[0]=='/') {
    /* set recovery point */
    tct->last_valid_tag = tag_start;
    return tag_start;
  } else {
    logmsg(LOG_INFO, "[%d] Tag starts with illegal characters.",
	   tct->tid);
    tct->error=UNALLOWED_CHAR;
    return NULL;
  }
}


/*
 * find_tagend
 *
 * Find end of tag.
 * Takes pointer to beginning of tag.
 * Result points right behind the tag or NULL if
 * tag contains illegal characters.
 */
static char *find_tagend(char *tag_start,TCT*tct) {
  int i=0; char *eot;

  if (check_inbuf_pointer(tag_start,tct)!=NO_ERROR) return NULL;

  /* Find end of legal characters. */
  while(legal_constituent(tag_start[i])) {
    i+=1;
  }

  eot=tag_start+i;
  if (check_inbuf_pointer(eot,tct)!=NO_ERROR) return NULL;

  /* tag ends with white space or '>', everything else indicates failure */
  if (legal_whitespace(eot[0]) || eot[0]=='>')
    return eot;
  else {
    logmsg(LOG_INFO, "[%d] Tag contains illegal characters",
	   tct->tid);
    tct->error=UNALLOWED_CHAR;
    return NULL;
  }
}
