/*
    Copyright (C) 2000 DFN-FWL (http://www.fwl.dfn.de/fwl)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
 * $Id: process_client.c,v 1.2 2001/03/01 16:02:08 glauschwuffel Exp $
 *
 * $Log: process_client.c,v $
 * Revision 1.2  2001/03/01 16:02:08  glauschwuffel
 * Adapted for the changes on read.c
 *
 * Revision 1.1  2000/06/19 22:06:33  glauschwuffel
 * Major overhaul of the HTML parser. Now we have something like a
 * state machine (yes, I know that I should've done it that way
 * in the first place ;)).
 *
 * Split up some modules (misc.c, copy_pkts.c). filter_html.c
 * is much more cleaner now. I think that everyone should be able
 * to understand how the HTML parser works now. Especially
 * the skipping of comments is easy to get now ;)
 *
 * This seems to be getting cleaner...
 *
 */
#include <sys/types.h>
#include <sys/socket.h> /* send() */
#include <errno.h>

#include "analyze_header.h"
#include "analyze_request_header.h"
#include "mythreads.h"
#include "log.h"
#include "read.h"
#include "misc.h" /* reset_buffers() */
#include "threads.h" /* check for hangup flag */

/*
 * process_client
 *
 * receive the header of the http-request. parse it
 * and modify it if necessary. then send it.
 */

int process_client(TCT *tct) {
  int result;
    
  reset_buffers(tct);   /* reset buffer offsets */

  /* yes, we have input */
  tct->more_input_data=YES;
      
  if (read_client_header(tct)==FAILURE) {
    if (check_for_hangup_flag(tct)==FAILURE) return FAILURE;
    
    /*
     * It's okay if we have no header. This is the case if the request
     * was generated manually (echo "GET..." | netcat ...)
     * Other errors fail and should generate an error page FIXME.
     */
    if (tct->error==NO_END_OF_HEADER) { /* we have no header */
      /* just log this fact */
      logmsg(LOG_INFO, "[%d] Client request contains no header.", tct->tid);
      logmsg(LOG_DEBUG, "[%d] Inbuf offset is %d of %d bytes.",
	     tct->tid, tct->inoffset, tct->inlen);
      /* continue as if there was an end of header */
    } else {
      return FAILURE;
    }
  }
  
  /*
   * Reading succeeded. Now analyze the inbuf and fill
   * the outbuffer.
   */
  result = analyze_request_header(tct);

  /* now the (new generated) header is in the output buffer */

  if (result==0) { /* there is no body */
    logmsg(LOG_INFO, "[%d] Client has no body", tct->tid);
  } else {
    /* process the body */
    if (tct->inoffset<tct->inlen) {
      /* until now the client body is just copied */
      logmsg(LOG_INFO, "[%d] Client bodies are just copied", tct->tid);
      output_string(tct->inbuf+tct->inoffset, tct->inlen-tct->inoffset,
		  tct);
    } else {
      logmsg(LOG_DEBUG, "[%d] Received empty body on client socket", tct->tid);
    }
  }
  
  /* send buffer */
  logmsg(LOG_INFO, "[%d] Sending client header containing %d bytes",
	 tct->tid, tct->outoffset);
  if (send(tct->outsocket, tct->outbuf, tct->outoffset, 0) != tct->outoffset) {
    logmsg(LOG_ERR, "[%d] Write error on outsocket %d with errno %d",
	   tct->tid, tct->outsocket,errno);
    return FAILURE;
  }
	  
  return OKAY;
}
