/*
    Copyright (C) 2000 DFN-FWL (http://www.fwl.dfn.de/fwl)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/* read the http.cfg and store the values into the hash tables
 */

/* $Id: read_config.c,v 1.16 2001/03/01 16:04:10 glauschwuffel Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>

#include "read_config.h"
#include "taghash.h"
#include "cfglists.h"
#include "ftype_check.h"
#include "log.h"

static int scan_attr_line(char*);
static int scan_http_line(char*);
static int scan_tag_line(char*);
static int scan_line(char*);

static int hextoi(int);
static int get_action(char*);

static int scan_attr_line(char *line) {
  char command_string[MAX_STRING_LENGTH] = {"\0"};
  char name_string[MAX_STRING_LENGTH] = {"\0"};
  char action_string[MAX_STRING_LENGTH] = {"\0"};
  int	action = 0;
  char replace_string[MAX_STRING_LENGTH] = {"\0"};
  int result = 0;

  result = sscanf(line, "%s %s %s %s", command_string, name_string,
  				  action_string, replace_string);
  if (result < 3) {
    logmsg(LOG_ERR, "Malformed attribute configuration: '%s'", line);
    return FAILURE;
  }

  logmsg(LOG_DEBUG, "Scanned attribute: %s :: %s :: %s :: %s", command_string,
	 name_string, action_string, replace_string);

  action = get_action(action_string);
  if (action == ACT_UNSPEC) {
    logmsg(LOG_ERR, "No action defined in config line: '%s'", line);
    return FAILURE;
  }

  /* insert attribute in hash-list */ 
  attrib_insert(name_string, replace_string, action);
  return OKAY;
}

static int scan_http_line(char *line) {
  return 0;
}

/* evaluate the string. Ascii-Characters are allowed and the quoted hex
   notation of c   '\x00'  */
static int eval_string(register char *source) {

  char  *sourcep;
  char	*tmp_point;
  char  destination[MAX_STRING_LENGTH];
  char	*destp;
  char	c;
  int   hex[2];

  destp = &destination[0];
  sourcep = source;
  while ((c = *sourcep++) != '\0') {
    if(c == '\\') {
        tmp_point = sourcep;
        c = *tmp_point++;
	if (c == 'x') {
	  hex[0] = hextoi(*tmp_point++);
          hex[1] = hextoi(*tmp_point++);
	  /* was it a hex digit? */
	  if ((hex[0] >= 0) && (hex[1] >= 0)) {
		*destp++ = (hex[0] << 4) + hex[1];
		sourcep = sourcep + 3;
	  } else {
	  	logmsg(LOG_ERR, "Error in HexQuote");
	  	return -1;
	  }
	} else {
	  *destp++ = '\\';
	  *destp++ = (char) c;
	  sourcep++;
	}
    } else {
      /* not quoted */
      *destp++ = (char)c;
    }
  }

  *destp = '\0';
  strncpy(source, &destination[0], MAX_STRING_LENGTH);
  return (destp - &destination[0]);
}

/* Convert single hex digit to int, return -1 on error */
static int hextoi(int c) {
  if (!isascii((unsigned char) c)) {
     return FAILURE; }
 
  if (isdigit((unsigned char) c)) {
     return (c - '0'); }
 
  if ((c>='a')&&(c<='f')) {
     return (c + 10 - 'a'); }

  if ((c>='A')&&(c<='F')) {
     return (c + 10 - 'A'); }

  return FAILURE;
}


static int scan_ftype_line(char *line) {
  char	command_string[MAX_STRING_LENGTH] = {"\0"};
  char	mimetype_string[MAX_STRING_LENGTH] = {"\0"};
  unsigned int	offset = 0;
  int	length = 0;
  char	search_string[MAX_STRING_LENGTH] = {"\0"};
  int	result = 0;

  result = sscanf(line, "%s %s %u %s", command_string, mimetype_string,
		  &offset, search_string);

  switch (result) {
  case 4:
    /* check pattern: */
    length = eval_string(&search_string[0]);
    if (length <= 0) {
      logmsg(LOG_ERR, "Search pattern for file type is malformed: '%s'", line);
      return FAILURE;
    }
    logmsg(LOG_DEBUG, "Scanned file type: %s :: %s :: %u :: %u :: %s", command_string,
	   mimetype_string, offset, length, search_string);
    type_insert(search_string, mimetype_string, offset, length);
    break;
  case 2:	
    logmsg(LOG_DEBUG, "Scanned file type: %s :: %s", command_string,
	   mimetype_string);
    type_insert(NULL, mimetype_string, 0, 0);
    break;
  default:
    logmsg(LOG_ERR, "Too few args for file type: '%s'", line);
    return FAILURE;
  }
  
  return OKAY;
}

/*
 * get_action
 *
 * Examine line and set action.
 * TODO: implement it with array and loop.
 */
static int get_action(char *line) {
  int  action;
  
  action = ACT_UNSPEC;
  if (strstr(line, "allow") != NULL) {
    action = action | ACT_ALLOW;
  }
  if (strstr(line, "replabort") != NULL) {
    action = action | ACT_REPLABORT;
  }
  if (strstr(line, "replcont") != NULL) {
    action = action | ACT_REPLCONT;
  }
  if (strstr(line, "log") != NULL) {
    action = action | ACT_LOG;
  }
  if (strstr(line, "replskip") != NULL) {
    action = action | ACT_REPLSKIP;
  }
  if (strstr(line, "replendskip") != NULL) {
    action = action | ACT_REPLENDSKIP;
  }

  return action;
}

static int scan_tag_line(char *line) {
  char command_string[MAX_STRING_LENGTH] = {"\0"};
  char name_string[MAX_STRING_LENGTH] = {"\0"};
  char action_string[MAX_STRING_LENGTH] = {"\0"};
  int	action = 0;
  char replace_string[MAX_STRING_LENGTH] = {"\0"};
  int result = 0;

  result = sscanf(line, "%s %s %s %s", command_string, name_string,
		  action_string, replace_string);
  if (result < 3) {
    logmsg(LOG_ERR, "Malformed tag configuration: '%s'", line);
    return FAILURE;
  }

  logmsg(LOG_DEBUG, "Scanned tag: %s :: %s :: %s :: %s", command_string,
	 name_string, action_string, replace_string);

  action = get_action(action_string);
  if (action == ACT_UNSPEC) {
    logmsg(LOG_ERR, "No action defined in config line: '%s'", line);
    return FAILURE;
  }

  /* insert <TAG> in hash-list */ 
  tag_insert(name_string, replace_string, action);
  return OKAY;
}

static int scan_protocol_line(char *line) {
  char command_string[MAX_STRING_LENGTH] = {"\0"};
  char proto_string[MAX_STRING_LENGTH] = {"\0"};
  int result;

  result = sscanf(line, "%s %s", command_string, proto_string);

  if (result < 2) {
    logmsg(LOG_ERR, "Malformed protocol configuration: '%s'", line);
    return FAILURE;
  }

  logmsg(LOG_DEBUG, "Scanned protocol: %s :: %s", command_string,
	 proto_string);

  /* insert protocol in hash-list */ 
  process_protos(proto_string);
  return OKAY;
}

static int scan_clients_line(char *line) {
  char command_string[MAX_STRING_LENGTH] = {"\0"};
  char clients_string[MAX_STRING_LENGTH] = {"\0"};
  int result;

  result = sscanf(line, "%s %s", command_string, clients_string);

  if (result < 2) {
    logmsg(LOG_ERR, "Malformed clients configuration: '%s'", line);
    return FAILURE;
  }

  logmsg(LOG_DEBUG, "Scanned clients: %s :: %s", command_string,
	 clients_string);

  /* insert protocol in hash-list */ 
  process_clients(clients_string);
  return OKAY;
}

static int scan_line(char *line) {
  char command_string[MAX_STRING_LENGTH];

  /* if sscanf returns 0, it must have been an empty line */
  if (sscanf(line, "%s ", command_string) == 1) {
    if (strncasecmp("tag", command_string, 4) == 0) {
      return scan_tag_line(line);
    }

    if (strncasecmp("attr", command_string, 5) == 0) {
      return scan_attr_line(line);
    }  

    if (strncasecmp("http", command_string, 5) == 0) {
      return scan_http_line(line);
    }  

    if (strncasecmp("ftype", command_string, 6) == 0) {
      return scan_ftype_line(line);
    }  

    if (strncasecmp("protocols", command_string, 10) == 0) {
      return scan_protocol_line(line);
    }  

    if (strncasecmp("clients", command_string, 8) == 0) {
      return scan_clients_line(line);
    }  

    /* comment line? */
    if (command_string[0] == '#') {
      return OKAY;
    }

    /* ok, the string was nothing we expected. so we print an error */
    logmsg(LOG_ERR, "Unknown configuration entry type: '%s'", line);
    logmsg(LOG_ERR, "Expected 'tag', 'attr', 'http', 'ftype' or 'protocols'");
    
    return FAILURE;
  }
  /* nothing was assigned by sscanf, so it must have been an empty line */
  return OKAY;
}

void read_config(char *filename) {
  FILE	*config_file;
  char	line[MAX_STRING_LENGTH];
  char	*line_ptr;
  int	line_count;
	
  config_file = fopen (filename,"r");
  if (config_file == NULL){
    logmsg(LOG_ERR, "Unable to open config-file: <%s> (with errno %d)", filename);
    exit(EXIT_FAILURE);
  }

  line_count = 0;

  do {
    line_ptr = fgets(line, MAX_STRING_LENGTH - 1, config_file);
    line_count++;
    scan_line(line);
  } while (line_ptr != NULL);

  if (line_count == 1) {
    logmsg(LOG_ERR, "Empty config-file: <%s>", filename);
    fclose(config_file);
    exit(EXIT_FAILURE);
  }

  logmsg(LOG_INFO, "Read config file, parsed %d lines", line_count);
  fclose(config_file);
}
