/*
 * $Id: signal.c,v 1.1 2001/03/01 16:07:27 glauschwuffel Exp $
 *
 * signal.c -- signal handler
 *
 * $Log: signal.c,v $
 * Revision 1.1  2001/03/01 16:07:27  glauschwuffel
 * Signal handler for SIGHUP. Open sockets are closed and the log is
 * closed and re-opened.
 *
 */

#include <stdlib.h>
#include <signal.h>

#include "threads.h"
#include "log.h"

struct sigaction signal_action, old_signal_action;

void hangup_handler(int signal) {

  logmsg(LOG_NOTICE, "Caught HANGUP signal");
  /* close the log so it can be zipped and put away */
  closelog();

  /* reopen it so we have log messages again ;) */
  openlog("httpf", LOG_PID, LOG_LOCAL6);

  /*
   * Now we close the file descriptor of each thread
   * that is working. The next/current recv() will
   * return with an error and processing of that
   * request stops. The user will have to press the
   * 'reload' button... what a burden ;)
   */
  close_descriptors();

  logmsg(LOG_NOTICE, "Processing of HANGUP signal finished");
}

void setup_signal_handler(void) {
  signal_action.sa_handler = hangup_handler;
  sigemptyset(&signal_action.sa_mask);
  signal_action.sa_flags=0;
  if (sigaction(SIGHUP,&signal_action,&old_signal_action) != 0) {
    logmsg(LOG_ERR, "Unable to install signal handler for SIGHUP");
    exit(EXIT_FAILURE);
  }
}
