/*
    Copyright (C) 2000 DFN-FWL (http://www.fwl.dfn.de/fwl)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 * $Id: taghash.c,v 1.9 2000/04/24 19:44:34 gellert Exp $
 *
 * $Log: taghash.c,v $
 * Revision 1.9  2000/04/24 19:44:34  gellert
 * Added new feature: clients entry in httpf.cfg. Only configured
 * clients are allowed to connect to httpf.
 * The handling of the lists of protocols and the lists
 * of allowed clients are now handled in cfglists.c
 *
 * Revision 1.8  2000/04/16 15:36:41  gellert
 * Just cleaned up a bit and removed some code that existed more
 * than once.
 *
 * Revision 1.7  2000/03/07 15:06:57  goldbach
 * added GPL info
 *
 * Revision 1.6  1999/08/19 14:31:08  gellert
 * enhanced checking of the protocols. now the ':' after the
 * protocol name is checked for.
 *
 * Revision 1.5  1999/07/28 14:15:10  gellert
 * Just some changes to the call of logmsg.
 * And removed old debugging statements.
 *
 * Revision 1.4  1999/07/16 09:45:39  goldbach
 * added include. there was an ugly warning :)
 *
 * Revision 1.3  1999/07/15 16:56:46  gellert
 * added protocol test for URLs. Nor allowed protocols are stored
 * in a list (eg. http,ftp,file). Other protocols are not allowed
 * (eg. javascript:, about:)
 *
 * Revision 1.2  1999/05/27 11:29:39  gellert
 * well, now it does what we want. just small mistakes (eg end of
 * comments) are done.
 *
 * Revision 1.1  1999/04/28 09:18:41  gellert
 * added the modules to read in the config file and place the
 * tags and attributes into a hash table.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>
#include <syslog.h>
#include <ctype.h>
#include "misc.h"
#include "taghash.h"

#define	TAG_SIZE	101
#define	ATTRIB_SIZE	101

struct hash_list {
    struct	hash_list *next;	/* next entry */
    char	*name;			/* name of the tag */
    char	*replace;		/* replacement for the tag */
    int		action;			/* what is to be done */
};

static struct hash_list *tag_list[TAG_SIZE];
static struct hash_list	*attrib_list[ATTRIB_SIZE];


/*
 * html_hash
 * Compute hash-value for a string.
 * Note: This is case-insensitive.
 */
unsigned html_hash(char *string, int tabsize) {
  unsigned hashval;
  
  for (hashval = 0; *string != '\0'; string++) {
  	hashval = tolower(*string) + 31 * hashval;
  	}
  return hashval % tabsize;
}


/* html_find: find given string in the hash_list */
struct hash_list *html_find(char *string, struct hash_list *htmllist[], int tabsize) {

  struct hash_list *hash_point;
  
  for (	hash_point = htmllist[html_hash(string, tabsize)];
  	hash_point != NULL;
	hash_point = hash_point->next) {
      
    if (strcasecmp(string, hash_point->name) == 0) {
    	return hash_point;
        } /* end of if */
    } /* end of for */
  return NULL;
}

/* html_replacement: return pointer to replacement string */
char *html_replacement(char *string, struct hash_list *table[], int tabsize) {
  struct hash_list *hash_point;
  
  hash_point = html_find(string, table, tabsize);

  if (hash_point != NULL) {
     return(hash_point->replace);
  } else {
     return (NULL);
  }
}

/* html_action: return action */
int html_action(char *string, struct hash_list *table[], int tabsize) {
  struct hash_list *hash_point;
  
  hash_point = html_find(string, table, tabsize);
  if (hash_point != NULL) {
     return(hash_point->action);
  } else {
     return (-1);
  }
}
  
/* html_insert: insert a tag into the hash table */
int html_insert(char *name, char *replacement, int action,
	struct hash_list *hashtab[], int tabsize) {

  struct   hash_list	*hash_point;
  unsigned		hashval;

  if ((hash_point = html_find(name, hashtab, tabsize)) == NULL) {	/* not found */
  	hash_point = (struct hash_list *) malloc(sizeof(*hash_point));
  	if ((hash_point == NULL) || ((hash_point->name = strdup(name)) == NULL)
  		|| (hash_point->replace = strdup(replacement)) == NULL) {
  		return (-1);
  		}
  	hash_point->action = action;
  	hashval = html_hash(name, tabsize);
  	hash_point->next = hashtab[hashval];
  	hashtab[hashval] = hash_point;
  	return 0;
  } else {					/* found */
	hash_point->action = action;
  	free((void *) hash_point->replace);	/* free old string */
	if ((hash_point->replace = strdup(replacement)) == NULL) {
		return (-1);
		}
  }
  return 0;	/* this is never reached, but to suppress compiler warnings */
}


/* tag_replacement: return pointer to replacement string */
char *tag_replacement(char *string) {
  return(html_replacement(string, tag_list, TAG_SIZE));
}

/* tag_action: return action */
int tag_action(char *string) {
  return(html_action(string, tag_list, TAG_SIZE));
}

/* tag_insert: insert a tag into the hash table */
int tag_insert(char *name, char *replacement, int action) {
    return(html_insert(name, replacement, action, tag_list, TAG_SIZE));
}  	


/* attrib_replacement: return pointer to replacement string */
char *attrib_replacement(char *string) {
  return(html_replacement(string, attrib_list, ATTRIB_SIZE));
}

/* attrib_action: return action */
int attrib_action(char *string) {
  return(html_action(string, attrib_list, ATTRIB_SIZE));
}

/* attrib_insert: insert an attribute into the hash table */
int attrib_insert(char *name, char *replacement, int action) {
    return(html_insert(name, replacement, action, attrib_list, ATTRIB_SIZE));
}  	

